@@ -7,8 +7,8 @@ C
 * Used by MRQMIN
 * H.Press et al - NUMERICAL RECEPIES, Cambridge UP 86, page 515
 *
-      DOUBLE PRECISION COVAR(NCVM,NCVM)
-      INTEGER LISTA(MFIT)
+      dimension COVAR(NCVM,NCVM)
+      dimension LISTA(MFIT)
 *
       DO  12  J=1,MA-1
         DO  11  I = J+1, MA
@@ -18,10 +18,13 @@ C
 *
       DO  14  I = 1, MFIT-1
         DO  13  J = I+1, MFIT
-          IF( LISTA(J) .GT. LISTA(I) ) THEN
-            COVAR(LISTA(J),LISTA(I)) = COVAR(I,J)
+c          IF( LISTA(J) .GT. LISTA(I) ) THEN
+c            COVAR(LISTA(J),LISTA(I)) = COVAR(I,J)
+           if( j.gt.i) then
+              covar( j,i ) =covar( i,j )
           ELSE
-            COVAR(LISTA(I),LISTA(J)) = COVAR(I,J)
+c            COVAR(LISTA(I),LISTA(J)) = COVAR(I,J)
+             covar( i, j ) = covar ( i, j )
           ENDIF
 13      CONTINUE
 14    CONTINUE
@@ -32,9 +35,13 @@ C
         COVAR(J,J) = 0.0
 15    CONTINUE
 *
-      COVAR(LISTA(1),LISTA(1)) = SWAP
+c      COVAR(LISTA(1),LISTA(1)) = SWAP
+       covar( 1 ,1 ) =swap
+c
       DO  16  J = 2, MFIT
-        COVAR(LISTA(J),LISTA(J)) = COVAR(1,J)
+c        COVAR(LISTA(J),LISTA(J)) = COVAR(1,J)
+          covar( j, j ) =covar( 1, j )
+
 16    CONTINUE
       DO  18  J = 2, MA
         DO  17  I = 1, J-1
