@@ -1,7 +1,6 @@
 C
 C
-C     *****************************************************************
-C
+C     **************************************************************
 C
 C     **************************************************************
 C
@@ -12,7 +11,7 @@ C     SEARCHES IPKSCH CHANNELS AROUND CENTIN VALUES TO GET
 C     BETTER GUESS FOR PEAK CENTROIDS
 C
       IMPLICIT DOUBLE PRECISION (A-H,O-Z)
-      DIMENSION SPEC(8192), CENTIN(30),GUESS(30)
+      DIMENSION SPEC(nchan), CENTIN(npeak),GUESS(30)
       DO 4001 I=1,NPEAK
       IMIN=CENTIN(I)-IPKSCH
       IMAX=CENTIN(I)+IPKSCH
