@@ -17,7 +17,15 @@
 
 #include <stdio.h>
 #include <math.h>
+#include <string.h>
 #include "symdefs.h"
+#include "alerts.h"
+#include "chersh.h"
+#include "ps.h"
+#include "xvlib.h"
+
+#define DECLARE
+#include "externs.h"
 
 extern int debuglevel;
 
@@ -51,13 +59,10 @@ static double curx, cury;	/* current location of pen */
 static int color;		/* current color */
 static int lines;		/* current linestyle */
 static int linew;		/* current line width */
-static int ylabpos;		/* info for positioning y-axis label */
 static int devtype;		/* current device */
 static int clipflag = 1;	/* clip in my_draw2 */
 static int scaletypex = NORMAL;	/* NORMAL, LOG, POLAR */
 static int scaletypey = NORMAL;	/* NORMAL, LOG, POLAR */
-static int swappedx = FALSE;	/* if axis is in reverse */
-static int swappedy = FALSE;	/* if axis is in reverse */
 
 extern int hardcopyflag;
 
@@ -65,23 +70,6 @@ extern int hardcopyflag;
  * globals. for device drivers
  */
 double devcharsize;		/* device charsize */
-int (*devsetcolor) ();		/* device set color */
-int (*devsetfont) ();		/* device set font */
-int (*devsetline) ();		/* device set line style */
-int (*devsetlinew) ();		/* device set line width */
-int (*devsetpat) ();		/* device set fill pattern */
-void (*devdrawarc) ();		/* device arc routine */
-void (*devfillarc) ();		/* device fill arc routine */
-void (*devdrawellipse) ();	/* device ellipse routine */
-void (*devfillellipse) ();	/* device ellipse arc routine */
-void (*devfill) ();		/* device fill polygon with a pattern */
-void (*devfillcolor) ();	/* device fill polygon with color */
-int (*devconvx) ();		/* viewport to device mapping for x */
-int (*devconvy) ();		/* viewport to device mapping for y */
-int (*vector) ();		/* device draw line */
-int (*devwritestr) ();		/* device write string */
-int (*devdrawtic) ();		/* device draw tic */
-int (*devleavegraphics) ();	/* device exit */
 int devsymsize;			/* device default symbol size */
 int devxticl;			/* device tic length for x-axis */
 int devyticl;			/* device tic length for y-axis */
@@ -105,9 +93,6 @@ static double hdelta;
 
 void my_move2();
 
-extern int stringextentx();	/* in chersh.c */
-extern int stringextenty();
-
 double xconv(), yconv();
 void device2view(), view2world();
 
@@ -266,7 +251,7 @@ double xconv(x)
 	if (x > 0.0) {
 	    x = log10(x);
 	} else {
-	    return 0;
+	    return 0.0;
 	}
     }
     return (rx * x + crx);
@@ -279,13 +264,13 @@ double yconv(y)
 	if (y > 0.0) {
 	    y = log10(y);
 	} else {
-	    return 0;
+	    return 0.0;
 	}
     }
     return (ry * y + cry);
 }
 
-int set_coordmap(mapx, mapy)
+void set_coordmap(mapx, mapy)
     int mapx, mapy;
 {
     scaletypex = mapx;
@@ -351,8 +336,8 @@ void defineworld(x1, y1, x2, y2, mapx, mapy)
 	break;
     }
     if (debuglevel == 3) {
-	printf("%lf %lf %lf %lf\n", x1, x2, y1, y2);
-	printf("%lf %lf %lf %lf\n", xg1, xg2, yg1, yg2);
+	printf("%f %f %f %f\n", x1, x2, y1, y2);
+	printf("%f %f %f %f\n", xg1, xg2, yg1, yg2);
     }
 }
 
@@ -383,6 +368,7 @@ void setclipping(fl)
     clipflag = fl;
 }
 
+#ifndef LIANG_BARSKY
 /*
  * clip lines within rectangle defined in world coordinates
  */
@@ -406,6 +392,7 @@ static int region(xend, yend)
     }
     return (endpoint);
 }
+#endif
 
 /*
  * liang-barsky line clipping, the default as of v2.05
@@ -480,8 +467,11 @@ static int clip2d(x0, y0, x1, y1)
 void my_draw2(x2, y2)
     double x2, y2;
 {
-    double m, minverse, x, y, x1, y1, xtmp, ytmp;
-    int dir, dir1, dir2, region();
+#ifndef LIANG_BARSKY
+    double m, minverse, x, y;
+    int dir, dir1, dir2;
+#endif
+    double x1, y1, xtmp, ytmp;
 
     x1 = curx;
     y1 = cury;
@@ -650,6 +640,9 @@ static int inside(x, y, which)
     case EDGE_RIGHT:
 	return (x <= rxg2);
     }
+    fprintf(stderr, "Unexpected parameter of which (%d) in routine inside.\n",
+	    which);
+    return 0;
 }
 
 static void intersect(px, py, x1, y1, x2, y2, which)
@@ -725,8 +718,10 @@ int which;
 	    px2[k] = px;
 	    py2[k] = py;
 	    k++;
+	/*
 	} else {
-	    /* do nothing */
+	    *** do nothing
+	*/
 	}
 	pxprev = px1[i];
 	pyprev = py1[i];
@@ -739,7 +734,7 @@ static void polygon_clip(px1, py1, n)
 int *n;
 
 {
-    int n1 = *n, n2, i;
+    int n1 = *n, n2;
     double px2[MAX_POLY], py2[MAX_POLY];
 
     clip_edge(px1, py1, px2, py2, n1, &n2, EDGE_ABOVE);
@@ -941,8 +936,9 @@ static void xhistbox(x, y)
     double tmpx[4];
     double tmpy[4];
     int i;
+/*
     int hist_pattern = 5;
-
+*/
     tmpx[0] = x - hdelta;
     tmpy[0] = 0.0;
     tmpx[1] = x - hdelta;
@@ -1246,12 +1242,10 @@ void writestr(x, y, dir, just, s)
 void drawtitle(title, which)
     char *title;
     int which;
-
 {
-    int fudge, ix, iy;
+    int /*ix, */iy;
     int tmp;
 
-    fudge = 4;
     tmp = ((*devconvx) (rxg2) + (*devconvx) (rxg1)) / 2;	/* center x-axis label
 								 * and title */
     if (title[0] && !which) {
@@ -1259,7 +1253,7 @@ void drawtitle(title, which)
     } else {
 	iy = stringextenty(charsize * devcharsize, "Xy");
     }
-    ix = stringextentx(charsize * devcharsize, title);
+/*    ix = stringextentx(charsize * devcharsize, title); */
     (*devwritestr) (tmp, (*devconvy) (ryg2) + iy, 0, title, 2, 0);
 /*
     if (hardcopyflag) {
@@ -1652,6 +1646,7 @@ void symdiamond(x, y, s, f)
     }
 }
 
+/*ARGSUSED*/
 void symplus(x, y, s, f)
     int x, y, f;
     double s;
@@ -1664,6 +1659,7 @@ void symplus(x, y, s, f)
     (*vector) (x, y - side, 1);
 }
 
+/*ARGSUSED*/
 void symx(x, y, s, f)
     int x, y, f;
     double s;
@@ -1676,6 +1672,7 @@ void symx(x, y, s, f)
     (*vector) (x + side, y - side, 1);
 }
 
+/*ARGSUSED*/
 void symstar(x, y, s, f)
     int x, y, f;
     double s;
@@ -1967,7 +1964,7 @@ void velplt(xx, yy, u, v, vscale)
     double u, v, vscale;
     double xx, yy;
 {
-    double x, y, s, xl, yl, xr, yr, adj;
+    double x, y, s, xl, yl, xr, yr;
     double theta, i1, i2;
     int ix, iy;
 
@@ -2047,7 +2044,7 @@ void putlegend(i, d, xlen, ylen, size, x, y, sy, ly, cy, wy, s, barflag, fill, f
     char *s;
 {
     int ipx[4], ipy[4], itmp, scol, slins, slinw, xtmp, ytmp;
-    static int maxx = 0, ifudgex, ifudgey;
+    static int ifudgex, ifudgey;
 
     xtmp = (*devconvx) (x);
     ytmp = (*devconvy) (y) - i * ylen * ifudgey;
@@ -2118,7 +2115,7 @@ void putlegend(i, d, xlen, ylen, size, x, y, sy, ly, cy, wy, s, barflag, fill, f
     (*devwritestr) (xtmp + (xlen + 1) * ifudgex, ytmp, 0, s, 0, 1);
 }
 
-putlegendrect(
+void putlegendrect(
 	      fill,
 	      fillusing,
 	      fillcolor,
