@@ -5,7 +5,21 @@
  */
 #include <stdio.h>
 #include <math.h>
+#include <string.h>
+#include <xview/xview.h>
 #include "globals.h"
+#include "alerts.h"
+#include "defaults.h"
+#include "draw.h"
+#include "events.h"
+#include "graphwin1.h"
+#include "labelwin.h"
+#include "main.h"
+#include "objutils.h"
+#include "setutils.h"
+#include "symwin.h"
+#include "tickwin.h"
+#include "worldwin.h"
 
 void flipxy();
 void invertx();
@@ -241,7 +255,7 @@ void get_graph_defaults(gno, d)
 void get_graph_framep(gno, f)
     int gno;
 
-    defaults *f;
+    framep *f;
 {
     memcpy(f, &g[gno].f, sizeof(framep));
 }
@@ -695,8 +709,7 @@ void defaulty(gno, setno)
     if (setno < 0) {
 	for (i = 0; i < g[gno].maxplot; i++) {
 	    if (isactive_set(gno, i)) {
-		if (g[gno].type == STACKEDBAR) {
-		} else {
+		if (g[gno].type != STACKEDBAR) {
 		    getsetminmax(gno, i, &tmp, &tmp, &ymin, &ymax);
 		    if (first) {
 			ygmin = ymin;
