@@ -40,6 +40,27 @@
 #include <xview/notify.h>
 
 #include "globals.h"
+#include "chersh.h"
+#include "compwin1.h"
+#include "compwin2.h"
+#include "defaults.h"
+#include "draw.h"
+#include "events.h"
+#include "fileswin.h"
+#include "graphutils.h"
+#include "graphutils2.h"
+#include "graphwin1.h"
+#include "labelwin.h"
+#include "printwin.h"
+#include "ps.h"
+#include "setutils.h"
+#include "setwin1.h"
+#include "symwin.h"
+#include "tickwin.h"
+#include "utils.h"
+#include "worldwin.h"
+#include "xvlib.h"
+
 #define SORT_TRUE   1
 #define SORT_FALSE  0
 #define SORT_FAIL  -1
@@ -52,29 +73,21 @@ static int sort_window = 0;
 int DBX_val = 0;
 
 int bc = 0;			/* for X11R4 servers w/XView 2.0 */
-extern void do_main_loop();
+void do_main_loop();
+Notify_value   destroy_func();
 
 void main(argc, argv)
     int argc;
     char *argv[];
 
 {
-    FILE *fp;
-    int i, j;
+    int i;
     int cur_graph = cg = 0;		/* default graph is graph 0 */
-    extern int realtime;	/* if data is to be plotted as it is read in */
     char *s;
-    int  ppid;                  /* process id of parent */
-    extern Notify_value   sort_sigusr1_handler();
-    extern Notify_value   destroy_func();
-    extern int shdat_init();
     
     s = getcwd(currentdir, 1024);	/* get the starting directory */
 
 
-    /* get parent process id */
-    ppid = getppid();
-    
     /* check options*/
     for(i=1; argv[i] != NULL; i++) {
 	  if (strcmp(argv[i],"-sort") == 0) {
@@ -91,7 +104,7 @@ void main(argc, argv)
     /*
      * open connection to  X server
      */
-    if (xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL) == NULL) {
+    if (xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL) == XV_NULL) {
 	fprintf(stderr, "xv_init failed, server problems?\n");
 	exit(1);
     }
@@ -106,8 +119,10 @@ void main(argc, argv)
     device = tdevice = 0;
 
     /* initialize the signal handler for SIGUSR1 and ignore ^C */
-    notify_set_signal_func(sort_client,sort_sigusr1_handler,SIGUSR1,NOTIFY_SYNC);
-    notify_set_signal_func(sort_client,SIG_DFL,SIGTERM,NOTIFY_SYNC);
+    notify_set_signal_func(sort_client,sort_sigusr1_handler,SIGUSR1,
+			   NOTIFY_SYNC);
+    notify_set_signal_func(sort_client,(Notify_value (*)()) SIG_DFL,
+			   SIGTERM,NOTIFY_SYNC);
     notify_set_destroy_func(sort_client, destroy_func);
 
 /* generate file name of SORT pipe for use with communications with sort process */
@@ -157,13 +172,10 @@ void main(argc, argv)
 #include <xview/frame.h>
 #include <xview/canvas.h>
 #include <xview/panel.h>
-#include <xview/cursor.h>
 #include <xview/font.h>
 #include <xview/svrimage.h>
 #include <xview/icon.h>
 #include <xview/cms.h>
-#include <xview/sel_attrs.h>
-#include <xview/textsw.h>
 
 /* paint window of canvas */
 Xv_Window paint_window;
@@ -353,7 +365,7 @@ void set_right_footer(s)
     char *s;
 {
     char *str;
-    struct tm tm, *localtime();
+    struct tm tm;
     long time_value;
     char name[256];
 
@@ -401,6 +413,8 @@ static void quit_main_proc(item, event)
     bailout();
 }
 
+#if 0 /* This function no longer used */
+
 /*
  * set action to SEL_POINT for selecting the locator reference point
  */
@@ -411,6 +425,9 @@ static void do_select_point()
     g[cg].pointset = TRUE;
 }
 
+#endif
+
+#if 0 /* This function no longer used */
 /*
  * clear the locator reference point
  */
@@ -420,6 +437,7 @@ static void do_clear_point()
     g[cg].pt_type = 0;
     g[cg].dsx = g[cg].dsy = 0.0;
 }
+#endif
 
 /*
  * total wipeout of graphs, sets, annotation
@@ -447,7 +465,7 @@ Panel_setting eotmark_proc(item, event)
     switch (event_id(event)) {
     case '\r':
     case '\n':
-	do_mark_proc(item, event);
+	do_mark_proc();
 	return PANEL_NONE;
     default:
 	return (panel_text_notify(item, event));
@@ -504,12 +522,12 @@ void set_curdir()
 void do_main_loop()
 {
     int i;
-    Menu m1, m2, m3, m4, m5;
+    Menu m1, m2 /*, m3, m4, m5*/;
     char name[256];
 
     getcwd(name, 255);
 
-    main_frame = (Frame) xv_create(NULL, FRAME,
+    main_frame = (Frame) xv_create(0, FRAME,
 				   FRAME_SHOW_FOOTER, TRUE,
 				   FRAME_LEFT_FOOTER, "Locator",
 				   FRAME_RIGHT_FOOTER, "Idle",
@@ -549,7 +567,7 @@ void do_main_loop()
 					 XV_HELP_DATA, "xvgr:mark",
 					 NULL);
 
-     m1 = (Menu) xv_create(NULL, MENU,
+     m1 = (Menu) xv_create(0, MENU,
 			  MENU_GEN_PIN_WINDOW, main_frame, "Autoscale",
 			  MENU_ITEM,
 			  MENU_STRING, "Autoscale",
@@ -653,7 +671,7 @@ void do_main_loop()
     }
     if (use_colors > 4) {
 	/* one more allocated than needed */
-	cms = (Cms) xv_create(NULL, CMS,
+	cms = (Cms) xv_create(0, CMS,
 			      CMS_NAME, "palette",
 			      CMS_TYPE, XV_DYNAMIC_CMS,
 			      CMS_SIZE, maxcolors + 1,
@@ -697,7 +715,7 @@ void do_main_loop()
     colors[0] = WhitePixel(disp, DefaultScreen(disp));
     colors[1] = BlackPixel(disp, DefaultScreen(disp));
     if (revflag) {
-	iswap(&colors[0], &colors[1]);
+	lswap(&colors[0], &colors[1]);
     }
     xv_set(canvas_paint_window(canvas), WIN_CONSUME_EVENTS,
 	   WIN_ASCII_EVENTS,
@@ -797,7 +815,7 @@ void do_main_loop()
 /*
  * create view pulldown menu
  */
-    m1 = (Menu) xv_create(NULL, MENU,
+    m1 = (Menu) xv_create(0, MENU,
 			  MENU_GEN_PIN_WINDOW, main_frame, "Graph ops",
 			  MENU_ITEM,
 			  MENU_STRING, "Activate...(1)",
@@ -841,7 +859,7 @@ void do_main_loop()
 		          XV_Y, xv_row(main_panel, 1),
 		          NULL); 
     
-/*    m2 = (Menu) xv_create(NULL, MENU,
+/*    m2 = (Menu) xv_create(0, MENU,
 			MENU_GEN_PIN_WINDOW, main_frame, "Strings & things",
 			  MENU_ITEM,
 			  MENU_STRING, "Text",
@@ -904,7 +922,7 @@ void do_main_loop()
 			  MENU_NOTIFY_PROC, do_clear_boxes, NULL,
 			  NULL);
 */
-/*    m3 = (Menu) xv_create(NULL, MENU,
+/*    m3 = (Menu) xv_create(0, MENU,
 			  MENU_ITEM,
 			  MENU_STRING, "Set fixed point",
 			  XV_HELP_DATA, "xvgr:fixedpoint_set",
@@ -1002,7 +1020,7 @@ void do_main_loop()
 /*
  * create calc(edit) pulldown
  */
-      m1 = (Menu) xv_create(NULL, MENU,
+      m1 = (Menu) xv_create(0, MENU,
    			  MENU_GEN_PIN_WINDOW, main_frame, "Calcs",
   			  MENU_ITEM,
 			  MENU_STRING, "Integration...",
@@ -1096,7 +1114,7 @@ void do_main_loop()
 			  MENU_NOTIFY_PROC, create_lconv_frame, NULL,  
 			  NULL);
 
-/*      m5 = (Menu) xv_create(NULL, MENU,
+/*      m5 = (Menu) xv_create(0, MENU,
 		     MENU_GEN_PIN_WINDOW, main_frame, "Pick set operations",
 			  MENU_ITEM, MENU_STRING,
 			  "", MENU_INACTIVE, TRUE, NULL, 
@@ -1130,7 +1148,7 @@ void do_main_loop()
 			  MENU_NOTIFY_PROC, do_delete_nearest, NULL,
 			  NULL); */
 
-   m2 = (Menu) xv_create(NULL, MENU,
+   m2 = (Menu) xv_create(0, MENU,
 			  MENU_GEN_PIN_WINDOW, main_frame, "Set operations",
 /*			  MENU_ITEM, MENU_STRING,
 			  "Pick operations",
@@ -1210,7 +1228,7 @@ void do_main_loop()
 			  MENU_NOTIFY_PROC, do_packsets, NULL, */
 			  NULL);
 
-/*    m4 = (Menu) xv_create(NULL, MENU,
+/*    m4 = (Menu) xv_create(0, MENU,
 			MENU_GEN_PIN_WINDOW, main_frame, "Point operations",
 			  MENU_ITEM,
 			  MENU_STRING, "Find points",
@@ -1301,12 +1319,12 @@ void do_main_loop()
 /*
  * init the icon
  */
-    closed_image = (Server_image) xv_create(NULL, SERVER_IMAGE,
+    closed_image = (Server_image) xv_create(0, SERVER_IMAGE,
 					    XV_WIDTH, 64,
 					    XV_HEIGHT, 64,
 					    SERVER_IMAGE_BITS, closed_bits,
 					    NULL);
-      mask_image = (Server_image) xv_create(NULL, SERVER_IMAGE,
+      mask_image = (Server_image) xv_create(0, SERVER_IMAGE,
 					  XV_WIDTH, 64,
 					  XV_HEIGHT, 64,
 					  SERVER_IMAGE_BITS, mask_bits,
@@ -1315,7 +1333,7 @@ void do_main_loop()
 /*
  * set the icon
  */
-    icon = (Icon) xv_create(NULL, ICON,
+    icon = (Icon) xv_create(0, ICON,
 			    ICON_IMAGE, closed_image,
 			    ICON_MASK_IMAGE, mask_image, 
 			    ICON_TRANSPARENT, TRUE, 
@@ -1388,6 +1406,7 @@ void update_draw()
 */    
 }
 
+/*ARGSUSED*/
 Notify_value
 destroy_func(client,sig,mode)
       Notify_client    client;
@@ -1396,4 +1415,7 @@ destroy_func(client,sig,mode)
 {
       if (DBX_val > 3) fprintf(stderr,"sort_xvgr: received SIGTERM\n");
       exit(0); 
+#ifdef lint
+      return 0;
+#endif
 }
