@@ -13,6 +13,11 @@
 #include <xview/panel.h>
 
 #include "globals.h"
+#include "alerts.h"
+#include "compwin1.h"
+#include "io.h"
+#include "main.h"
+#include "setutils.h"
 
 char format[128] = "%lg %lg";	/* for write sets */
 char sformat[128] = "%lg %lg";	/* for save all */
@@ -20,19 +25,14 @@ char sformat[128] = "%lg %lg";	/* for save all */
 #define FRAME_KEY        101
 void generic_done_proc();
 
-extern Frame main_frame;
-
 static char dirbuf[MAXPATHLEN];
-static char errbuf[256];
 
 extern int index_set_types[];
 extern int index_set_ncols[];
-extern void drawgraph2();
 
 /**********************************/
 static Panel_item activate_set_item;
 static Panel_item alength_to_set_item;
-static Panel_item set_length_item;
 static Panel_item set_settype_item;
 
 static void do_setlength_proc();
@@ -274,7 +274,6 @@ void create_swap_frame()
 static void do_copy_proc()
 {
     int j1, j2, gto, gfrom;
-    char buf[32];
 
     j1 = (int) xv_get(swap_from_item, PANEL_VALUE);
     gfrom = (int) xv_get(swap_fgraph_item, PANEL_VALUE);
@@ -432,7 +431,7 @@ void create_kill_frame()
  */
 static void do_kill_proc()
 {
-    int setno, soft, redraw = 0, i;
+    int setno, soft;
 
     setno = (int) xv_get(kill_set_item, PANEL_VALUE);
     soft = (int) xv_get(kill_soft_toggle, PANEL_VALUE);
@@ -503,7 +502,7 @@ void create_sort_frame()
 */
 static void do_sort_proc()
 {
-    int setno, sorton, stype, i;
+    int setno, sorton, stype;
 
     setno = (int) xv_get(sort_set_item, PANEL_VALUE);
     if (setno == MAXPLOT) {
@@ -601,6 +600,7 @@ void create_write_frame()
 /*
  *  write a set or sets to a file
  */
+/*ARGSUSED*/
 static Panel_setting do_write_sets_proc(item, event)
     Panel_item item;
     Event *event;
