@@ -11,11 +11,12 @@
 #include <xview/canvas.h>
 #include <xview/panel.h>
 #include <xview/font.h>
-#include <xview/cursor.h>
 
 #include "globals.h"
-
-extern Frame main_frame;
+#include "alerts.h"
+#include "events.h"
+#include "graphutils.h"
+#include "main.h"
 
 Frame ticks_frame = (Frame) 0;
 Panel ticks_panel;
@@ -30,7 +31,6 @@ static Panel_item altmax;	/* alternate mapping for axis */
 static Panel_item tonoff;	/* toggle display of axis ticks */
 static Panel_item tlonoff;	/* toggle display of tick labels */
 static Panel_item axislabel;	/* axis label */
-static Panel_item axislabelop;	/* axis label on opposite side */
 static Panel_item axislabellayout;	/* axis label layout (perp or
 					 * parallel) */
 static Panel_item axislabelfont;/* axis label font */
@@ -49,11 +49,7 @@ static Panel_item tlfont;	/* tick label font */
 static Panel_item tlcharsize;	/* tick label charsize */
 static Panel_item tlcolor;	/* tick label color */
 static Panel_item tllinew;	/* tick label color */
-static Panel_item tlvgap;	/* */
-static Panel_item tlhgap;	/* */
 static Panel_item tlskip;	/* tick marks to skip */
-static Panel_item tltype;	/* tick label type (auto or specified) */
-static Panel_item ttype;	/* tick mark type (auto or specified) */
 static Panel_item tlstarttype;	/* use graph min or starting value */
 static Panel_item tlstart;	/* value to start tick labels */
 static Panel_item tlstoptype;	/* use graph max or stop value */
@@ -66,9 +62,7 @@ static Panel_item tlsign;	/* sign of tick label (normal, negate, *
 				 * absolute) */
 static Panel_item tlappend;	/* append string to tick labels */
 static Panel_item tlprepend;	/* prepend string to tick labels */
-static Panel_item tlspec;	/* tick labels specified */
 static Panel_item tick_applyto;	/* apply to several axes */
-static Panel_item tnum;		/* number of ticks for autoscaling */
 static Panel_item tgrid;	/* major ticks grid */
 static Panel_item tgridcol;
 static Panel_item tgridlinew;
@@ -80,7 +74,6 @@ static Panel_item tmgridlines;
 static Panel_item tlen;		/* tick length */
 static Panel_item tmlen;
 static Panel_item tinout;	/* ticks in out or both */
-static Panel_item tspec;	/* tick marks specified */
 static Panel_item baronoff;	/* axis bar */
 static Panel_item barcolor;
 static Panel_item barlinew;
@@ -433,13 +426,13 @@ static void ticks_define_notify_proc()
 
 static Frame axis_frame;
 static Panel axis_panel; 
-static Panel_item axis_applyto; 
 
-static props_Done_notify_proc()
+static void props_Done_notify_proc()
 {
     xv_set(axis_frame, WIN_SHOW, FALSE, 0);
 }
 
+/*ARGSUSED*/
 static void accept_axis_proc(w)
     Panel_item w;
 {
@@ -555,9 +548,8 @@ static void do_axis_proc()
 
 static Frame axislabel_frame;
 static Panel axislabel_panel;
-static Panel_item axislabel_applyto;
 
-static axislabel_Done_notify_proc()
+static void axislabel_Done_notify_proc()
 {
     xv_set(axislabel_frame, WIN_SHOW, FALSE, 0);
 }
@@ -704,7 +696,7 @@ static void do_axislabel_proc()
 static Panel_item ticklabel_frame;
 static Panel_item ticklabel_panel;
 
-static ticklabel_Done_notify_proc()
+static void ticklabel_Done_notify_proc()
 {
     xv_set(ticklabel_frame, WIN_SHOW, FALSE, 0);
 }
@@ -822,14 +814,14 @@ static void update_ticklabel_items(gno)
 	xv_set(tlstagger, PANEL_VALUE, t.tl_staggered, NULL);
 	xv_set(tlstarttype, PANEL_VALUE, t.tl_starttype == SPEC, NULL);
 	if (t.tl_starttype == SPEC) {
-	    sprintf(buf, "%lf", t.tl_start);
+	    sprintf(buf, "%f", t.tl_start);
 	    xv_set(tlstart, PANEL_VALUE, buf, NULL);
-	    sprintf(buf, "%lf", t.tl_stop);
+	    sprintf(buf, "%f", t.tl_stop);
 	    xv_set(tlstop, PANEL_VALUE, buf, NULL);
 	}
 	xv_set(tlstoptype, PANEL_VALUE, t.tl_stoptype == SPEC, NULL);
 	if (t.tl_stoptype == SPEC) {
-	    sprintf(buf, "%lf", t.tl_stop);
+	    sprintf(buf, "%f", t.tl_stop);
 	    xv_set(tlstop, PANEL_VALUE, buf, NULL);
 	}
 	xv_set(tlappend, PANEL_VALUE, t.tl_appstr, NULL);
@@ -1136,7 +1128,7 @@ static void do_ticklabels_proc()
 static Panel_item tickmark_frame;
 static Panel_item tickmark_panel;
 
-static tickmark_Done_notify_proc()
+static void tickmark_Done_notify_proc()
 {
     xv_set(tickmark_frame, WIN_SHOW, FALSE, 0);
 }
@@ -1458,11 +1450,12 @@ static Frame axisbar_frame;
 static Panel axisbar_panel;
 static Panel_item axisbar_applyto;
 
-static axisbar_Done_notify_proc()
+static void axisbar_Done_notify_proc()
 {
     xv_set(axisbar_frame, WIN_SHOW, FALSE, 0);
 }
 
+/*ARGSUSED*/
 static void accept_axisbar_proc(w)
     Panel_item w;
 {
@@ -1518,9 +1511,6 @@ static void update_axisbar_items(gno)
 
 static void do_axisbar_proc()
 {
-    int i;
-    char buf[128];
-
     if (axisbar_frame) {
 	update_axisbar_items(cg);
 	xv_set(axisbar_frame, WIN_SHOW, TRUE, 0);
@@ -1604,10 +1594,7 @@ static Frame special_frame;
 static Panel special_panel;
 static Panel_item spec_applyto;
 
-static char savelab[MAX_TICK_LABELS][80];
-static char saveloc[MAX_TICK_LABELS][30];
-
-static special_Done_notify_proc()
+static void special_Done_notify_proc()
 {
     xv_set(special_frame, WIN_SHOW, FALSE, 0);
 }
@@ -1671,13 +1658,12 @@ static void update_special_items(gno)
     int gno;
 {
     tickmarks t;
-    int i, j, itmp;
+    int i, itmp;
 
     if (special_frame) {
 	get_graph_tickmarks(gno, &t, curaxis);
 	xv_set(specticks, PANEL_VALUE, t.t_type == SPEC, NULL);
 	xv_set(specticklabels, PANEL_VALUE, t.tl_type == SPEC, NULL);
-	j = 0;
 	if (tcurpage == 0) {
 	    itmp = (NPAGES - 1) * TPAGESIZE;
 	} else {
@@ -1704,7 +1690,7 @@ static void load_special(gno, a)
 	sprintf(buf, "%d", t.t_spec);
 	xv_set(nspec, PANEL_VALUE, buf, NULL);
 	for (i = 0; i < t.t_spec; i++) {
-	    sprintf(buf, "%lf", t.t_specloc[i]);
+	    sprintf(buf, "%f", t.t_specloc[i]);
 	    xv_set(specloc_item[i], PANEL_VALUE, buf, NULL);
 	    xv_set(speclabel_item[i], PANEL_VALUE, t.t_speclab[i].s, NULL);
 	}
