@@ -10,7 +10,6 @@
 
 #include <X11/Xlib.h>
 #include <xview/xview.h>
-#include <xview/canvas.h>
 #include <xview/cms.h>
 
 #endif
@@ -27,6 +26,10 @@
 
 #include "externs.h"
 #include "patterns.h"
+#include "chersh.h"
+#include "draw.h"
+#include "events.h"
+#include "main.h"
 
 /* external variables */
 extern Display *disp;
@@ -42,7 +45,6 @@ extern int revflag;		/* defined in main.c */
 extern int inwin;
 
 extern double devcharsize;
-extern double charsize;
 
 #define NUM_COLORS 256
 
@@ -107,6 +109,7 @@ void drawxlib();
 void xlibdoublebuffer();
 void xlibinit_tiles();
 
+/*ARGSUSED*/
 void get_xlib_dims(w, h)
     int *w, *h;
 {
@@ -127,7 +130,7 @@ static void xlibinit()
 {
     double wx1, wx2, wy1, wy2;
     static int inc = 1;
-    extern int invert, overlay, doclear, bgcolor;
+    extern int invert;
 
 #ifdef MOTIF
     Arg args;
@@ -196,7 +199,6 @@ void xclear_region(x,y,width,height)
       int x,y;
       int width, height;
 {
-      extern int bgcolor;
       xlibsetcolor(bgcolor);
       XFillRectangle(disp, displaybuff, gc, x, y, width, height);
       return;
@@ -219,7 +221,7 @@ void xlibdoublebuffer(mode)
     } else {
 	if (backbuff) {
 	    XFreePixmap(disp, backbuff);
-	    backbuff = NULL;
+	    backbuff = 0;
 	    displaybuff = xwin;
 	}
     }
@@ -332,8 +334,6 @@ static int dash_list_length[] = {1, 2, 2, 2, 4};
 
 static void flush_pending()
 {
-    int i;
-
     if (npending > 1) {
 	if (xlib_write_mode) {
 	    XDrawLines(disp, displaybuff, gc, polypoints, npending, CoordModeOrigin);
@@ -640,6 +640,7 @@ void xlibdrawtic(x, y, dir, updown)
     }
 }
 
+/*ARGSUSED*/
 void dispstrxlib(x, y, rot, s, just, fudge)
     int x, y, rot, just, fudge;
     char *s;
@@ -667,7 +668,7 @@ void xlibinit_tiles()
 	tiles[i] = XCreatePixmap(disp, xwin, 16, 16, DisplayPlanes(disp, DefaultScreen(disp)));
     }
     for (i = 0; i < MAXPATTERNS; i++) {
-	if (tiles[i] == NULL) {
+	if (tiles[i] == 0) {
 	    printf("bad tile %d\n", i);
 	} else {
 	    XFillRectangle(disp, tiles[i], gcclr, 0, 0, 16, 16);
@@ -738,6 +739,7 @@ int xlibsetpat(k)
     if (patno != 0) {
 	curtile = tiles[k - 1];
     }
+    return k;
 }
 
 void xlibfill(n, px, py)
@@ -745,7 +747,7 @@ void xlibfill(n, px, py)
     int px[], py[];
 
 {
-    int i, x, y;
+    int i;
     XPoint *p;
 
     p = (XPoint *) calloc(n, sizeof(XPoint));
@@ -774,7 +776,7 @@ void xlibfillcolor(n, px, py)
     int px[], py[];
 
 {
-    int i, x, y;
+    int i;
     XPoint *p;
 
     p = (XPoint *) calloc(n, sizeof(XPoint));
@@ -885,6 +887,7 @@ static Cursor kill_cursor;
 static int cur_cursor = -1;
 void set_cursor();
 
+/*ARGSUSED*/
 void set_wait_cursor(w)
 #ifdef MOTIF
     Widget w;
@@ -903,6 +906,7 @@ void set_wait_cursor(w)
 #endif
 }
 
+/*ARGSUSED*/
 void unset_wait_cursor(w)
 #ifdef MOTIF
     Widget w;
