@@ -4,17 +4,13 @@
 #include <xview/xview.h>
 #include <xview/canvas.h>
 #include <xview/panel.h>
-#include <xview/scrollbar.h>
 #include <xview/svrimage.h>
-#include <xview/termsw.h>
-#include <xview/text.h>
-#include <xview/tty.h>
-#include <xview/xv_xrect.h>
 #include "spec_ui.h"
 
 /*
  * Create object `palettes' in the specified instance.
  */
+/*ARGSUSED*/
 Xv_opaque
 spec_palettes_create(ip, owner)
 	caddr_t		ip;
@@ -80,6 +76,7 @@ spec_palettes_create(ip, owner)
 /*
  * Create object `levelMenu' in the specified instance.
  */
+/*ARGSUSED*/
 Xv_opaque
 spec_levelMenu_create(ip, owner)
 	caddr_t		ip;
@@ -128,6 +125,7 @@ spec_levelMenu_create(ip, owner)
 /*
  * Create object `windowMenu' in the specified instance.
  */
+/*ARGSUSED*/
 Xv_opaque
 spec_windowMenu_create(ip, owner)
 	caddr_t		ip;
@@ -187,6 +185,7 @@ spec_windowMenu_create(ip, owner)
 /*
  * Create object `autoscaleMenu' in the specified instance.
  */
+/*ARGSUSED*/
 Xv_opaque
 spec_autoscaleMenu_create(ip, owner)
 	caddr_t		ip;
@@ -216,6 +215,7 @@ spec_autoscaleMenu_create(ip, owner)
 /*
  * Create object `magnifyMenu' in the specified instance.
  */
+/*ARGSUSED*/
 Xv_opaque
 spec_magnifyMenu_create(ip, owner)
 	caddr_t		ip;
@@ -514,6 +514,7 @@ spec_specWindow_sliceButton_create(ip, owner)
 /*
  * Create object `slicingMenu' in the specified instance.
  */
+/*ARGSUSED*/
 Xv_opaque
 spec_slicingMenu_create(ip, owner)
 	caddr_t		ip;
@@ -1938,7 +1939,7 @@ spec_projectPopup_projectArea_create(ip, owner)
 	spec_projectPopup_objects	*ip;
 	Xv_opaque	owner;
 {
-    extern Panel_setting spec_projectPopup_projectArea_notify_callback();
+    extern void spec_projectPopup_projectArea_notify_callback();
     Xv_opaque obj;
 	
     obj = xv_create(owner, PANEL_CHOICE, PANEL_DISPLAY_LEVEL, PANEL_CURRENT,
@@ -2092,7 +2093,7 @@ spec_projectPopup_projectAxis_create(ip, owner)
 	spec_projectPopup_objects	*ip;
 	Xv_opaque	owner;
 {
-    extern Panel_setting spec_projectPopup_projectAxis_notify_callback();
+    extern void spec_projectPopup_projectAxis_notify_callback();
     Xv_opaque obj;
 	
     obj = xv_create(owner, PANEL_CHOICE, PANEL_DISPLAY_LEVEL, PANEL_CURRENT,
