@@ -6,7 +6,6 @@
  */
 
 #include <stdio.h>
-#include <stdlib.h>
 #include "files.h"
 
 /*
@@ -19,7 +18,7 @@ char *mygetline(char *buffer, int buflen, FILE *fp)
     do {
 	while((c = getc(fp)) != EOF && c != '\n')
 	{
-	    if (c == '*' || c == '#')
+	    if (c == '*' || c == '#' || c == '!')
 	    {
 		while((c = getc(fp)) != EOF && c != '\n')
 		    ;
