@@ -4,6 +4,8 @@
 
 char *mygetline(char *,int, FILE *); /*ignoring comments & blanks*/
 
+#define MYGETLINE(b, s, f, mess, rv) \
+    {if (mygetline(b, s, f) == NULL) {fprintf(stderr, mess); return rv;}}
 
 
 
