@@ -3,7 +3,8 @@
 extern struct reac { /*Reaction Parameters also used in fortran common block*/
     float edisp;
     float e1;
-    float mass[5];
+    float mass[/*5*/13];
+    float q1;
 } reac;
 
 int reac_init(void);          /*reaction definition*/
