@@ -6,8 +6,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <memory.h>
-#include <errno.h>
 
 #include <sys/stat.h>
 #include <sys/param.h>
@@ -15,145 +13,39 @@
 
 #include <xview/xview.h>
 #include <xview/canvas.h>
-#include <xview/icon_load.h>
 #include <xview/panel.h>
-#include <xview/scrollbar.h>
 #include <xview/svrimage.h>
-#include <xview/termsw.h>
-#include <xview/text.h>
-#include <xview/tty.h>
-#include <xview/xv_xrect.h>
 
+#include "sort_def.h"
+#include "sort_thread.h"
+#include "gui_alerts.h"
+#include "gui_main.h"
+#include "gui_notify.h"
+#include "gui_spectra.h"
+#include "gui_specpop.h"
+#include "gui_winpop.h"
 #include "gui_extern.h"
 
-extern Xv_opaque
-sort_gui_spec_popup_spec_popup_create(
-#if NeedFunctionPrototype
-				     Xv_opaque
-#endif
-				      );
-
-extern Xv_opaque
-sort_gui_spec_popup_control_spec_create(
-#if NeedFunctionPrototype
-					Xv_opaque
-#endif
-					);
-
-extern Xv_opaque
-sort_gui_spec_popup_spec_message1_create(
-#if NeedFunctionPrototype
-					 Xv_opaque
-#endif
-					 );
-
-extern Xv_opaque	
-sort_gui_spec_popup_spec1d_list_create(
-#if NeedFunctionPrototype
-				       Xv_opaque
-#endif
-				       );
-
-extern Xv_opaque
-sort_gui_spec_popup_spec1d_b1_create(
-#if NeedFunctionPrototype
-				     Xv_opaque
-#endif
-				     );
-
-extern Xv_opaque
-sort_gui_spec_popup_spec1d_b2_create(
-#if NeedFunctionPrototype
-				     Xv_opaque
-#endif
-				     );
-
-extern Xv_opaque
-sort_gui_spec_popup_spec1d_bn_create(
-#if NeedFunctionPrototype
-				     Xv_opaque
-#endif
-				     );
-
-extern Xv_opaque
-sort_gui_spec_popup_spec1d_b3_create(
-#if NeedFunctionPrototype
-				     Xv_opaque
-#endif
-				     );
-
-extern Xv_opaque
-sort_gui_spec_popup_spec1d_b4_create(
-#if NeedFunctionPrototype
-				     Xv_opaque
-#endif
-				     );
-
-extern Xv_opaque
-sort_gui_spec_popup_spec_clearall_but_create(
-#if NeedFunctionPrototype
-					     Xv_opaque
-#endif
-					     );
-
-extern Xv_opaque
-sort_gui_spec_popup_spec_message2_create(
-#if NeedFunctionPrototype
-					 Xv_opaque
-#endif
-					 );
-
-extern Xv_opaque
-sort_gui_spec_popup_spec2d_list_create(
-#if NeedFunctionPrototype
-				       Xv_opaque
-#endif
-				       );
-
-extern Xv_opaque
-sort_gui_spec_popup_spec2d_b1_create(
-#if NeedFunctionPrototype
-				     Xv_opaque
-#endif
-				     );
-extern Xv_opaque
-sort_gui_spec_popup_spec2d_bn_create(
-#if NeedFunctionPrototype
-				     Xv_opaque
-#endif
-				     );
-extern Xv_opaque
-sort_gui_spec_popup_spec2d_b2_create(
-#if NeedFunctionPrototype
-				     Xv_opaque
-#endif
-				     );
-
-extern Xv_opaque
-sort_gui_spec_popup_spec2d_b3_create(
-#if NeedFunctionPrototype
-				     Xv_opaque
-#endif
-				     );
-extern Xv_opaque
-sort_gui_spec_popup_spec2d_b4_create(
-#if NeedFunctionPrototype
-				     Xv_opaque
-#endif
-				     );
-
-extern Xv_opaque
-sort_gui_spec_popup_spec2d_b5_create(
-#if NeedFunctionPrototype
-				     Xv_opaque
-#endif
-				     );
-extern Xv_opaque
-sort_gui_spec_popup_spec2d_b6_create(
-#if NeedFunctionPrototype
-				     Xv_opaque
-#endif
-				     );
+Xv_opaque sort_gui_spec_popup_spec_popup_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_control_spec_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec_message1_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec1d_list_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec1d_b1_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec1d_b2_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec1d_bn_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec1d_b3_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec1d_b4_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec1d_baction_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec_clearall_but_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec_message2_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec2d_list_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec2d_b1_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec2d_bn_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec2d_b2_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec2d_b3_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec2d_b4_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec2d_b5_create P((Xv_opaque));
+Xv_opaque sort_gui_spec_popup_spec2d_b6_create P((Xv_opaque));
 
 /*
  * Initialize an instance of object `spec_popup'.
@@ -189,6 +81,8 @@ sort_gui_spec_popup_objects_initialize(
 		ip->spec1d_b3 = sort_gui_spec_popup_spec1d_b3_create(ip->control_spec);
 	if (!ip->spec1d_b4)
 		ip->spec1d_b4 = sort_gui_spec_popup_spec1d_b4_create(ip->control_spec);
+	if (!ip->spec1d_baction)
+		ip->spec1d_baction = sort_gui_spec_popup_spec1d_baction_create(ip->control_spec);
 	if (!ip->spec_clearall_but)
 		ip->spec_clearall_but = sort_gui_spec_popup_spec_clearall_but_create(ip->control_spec);
 	if (!ip->spec_message2)
@@ -330,11 +224,9 @@ sort_gui_spec_popup_spec1d_list_create(
       Xv_opaque	owner;
 #endif
 {
-#if NeedFunctionPrototype
-      extern int	spec1d_list_nh(Panel_item, char *, Xv_opaque, Panel_list_op, Event *);
-#else
-
-      extern int	spec1d_list_nh();
+#ifdef SUNSORT_DEBUG
+      extern int spec1d_list_nh P((Panel_item, char *, Xv_opaque,
+	  Panel_list_op, Event *));
 #endif
       Xv_opaque	obj;
       
@@ -396,11 +288,7 @@ sort_gui_spec_popup_spec1d_b1_create(
       Xv_opaque	owner;
 #endif
 {
-#if NeedFunctionPrototype
-      extern void		spec1d_b1_nh(Panel_item, Event *);
-#else
-      extern void		spec1d_b1_nh();
-#endif
+      extern void spec1d_b1_nh P((Panel_item, Event *));
 	Xv_opaque	obj;
 	
 	obj = xv_create(owner, PANEL_BUTTON,
@@ -428,11 +316,7 @@ sort_gui_spec_popup_spec1d_b2_create(
       Xv_opaque	owner;
 #endif
 {
-#if NeedFunctionPrototype
-      extern void		spec1d_b1_nh(Panel_item, Event *);
-#else
-      extern void		spec1d_b1_nh();
-#endif
+      extern void spec1d_b1_nh P((Panel_item, Event *));
 	Xv_opaque	obj;
 	
 	obj = xv_create(owner, PANEL_BUTTON,
@@ -459,11 +343,7 @@ sort_gui_spec_popup_spec1d_bn_create(
       Xv_opaque	owner;
 #endif
 {
-#if NeedFunctionPrototype
-      extern void		spec1d_b1_nh(Panel_item, Event *);
-#else
-      extern void		spec1d_b1_nh();
-#endif
+      extern void spec1d_b1_nh P((Panel_item, Event *));
 	Xv_opaque	obj;
 	
 	obj = xv_create(owner, PANEL_BUTTON,
@@ -491,11 +371,7 @@ sort_gui_spec_popup_spec1d_b3_create(
       Xv_opaque	owner;
 #endif
 {
-#if NeedFunctionPrototype
-      extern void		spec1d_b3_nh(Panel_item, Event *);
-#else
-      extern void		spec1d_b3_nh();
-#endif
+      extern void spec1d_b3_nh P((Panel_item, Event *));
 	Xv_opaque	obj;
 	
 	obj = xv_create(owner, PANEL_BUTTON,
@@ -523,11 +399,7 @@ sort_gui_spec_popup_spec1d_b4_create(
       Xv_opaque	owner;
 #endif
 {
-#if NeedFunctionPrototype
-      extern void		spec1d_b4_nh(Panel_item, Event *);
-#else
-      extern void		spec1d_b4_nh();
-#endif
+      extern void spec1d_b4_nh P((Panel_item, Event *));
 	Xv_opaque	obj;
 	
 	obj = xv_create(owner, PANEL_BUTTON,
@@ -542,6 +414,35 @@ sort_gui_spec_popup_spec1d_b4_create(
 	return obj;
 }
 
+/*
+ * Create object `spec1d_baction' in the specified instance.
+
+ */
+Xv_opaque
+sort_gui_spec_popup_spec1d_baction_create
+#if NeedFunctionPrototype
+(Xv_opaque	owner)
+#else
+(owner)
+      Xv_opaque	owner;
+#endif
+{
+      extern void spec1d_baction_nh P((Panel_item, Event *));
+	Xv_opaque	obj;
+	
+	obj = xv_create(owner, PANEL_CHOICE_STACK,
+			XV_X, 10,
+			XV_Y, 209,
+			PANEL_LABEL_WIDTH, 0,
+			XV_HEIGHT, 19,
+			PANEL_LABEL_STRING, "",
+			PANEL_NOTIFY_PROC, spec1d_baction_nh,
+			PANEL_CHOICE_NCOLS, 1,
+			PANEL_CHOICE_STRINGS, "Display", "Mstick", "Pkfind", "Paper", NULL,
+		NULL);
+	return obj;
+}
+
 /*
  * Create object `spec_clearall_but' in the specified instance.
 
@@ -555,11 +456,7 @@ sort_gui_spec_popup_spec_clearall_but_create(
       Xv_opaque	owner;
 #endif
 {
-#if NeedFunctionPrototype
-      extern void		spec_clearall_nh(Panel_item, Event *);
-#else
-      extern void		spec_clearall_nh();
-#endif
+      extern void spec_clearall_nh P((Panel_item, Event *));
       Xv_opaque	obj;
       
       obj = xv_create(owner, PANEL_BUTTON,
@@ -613,10 +510,9 @@ sort_gui_spec_popup_spec2d_list_create(
       Xv_opaque	owner;
 #endif
 {
-#if NeedFunctionPrototype
-      extern int	spec2d_list_nh(Panel_item, char *, Xv_opaque, Panel_list_op, Event *);
-#else
-      extern int	spec2d_list_nh();
+#ifdef SUNSORT_DEBUG
+      extern int spec2d_list_nh P((Panel_item, char *, Xv_opaque,
+	  Panel_list_op, Event *));
 #endif
 	Xv_opaque	obj;
 	
@@ -674,11 +570,7 @@ sort_gui_spec_popup_spec2d_b1_create(
       Xv_opaque	owner;
 #endif
 {
-#if NeedFunctionPrototype
-      extern void		spec2d_b1_nh(Panel_item, Event *);
-#else
-      extern void		spec2d_b1_nh();
-#endif
+      extern void spec2d_b1_nh P((Panel_item, Event *));
 	Xv_opaque	obj;
 	
 	obj = xv_create(owner, PANEL_BUTTON,
@@ -705,11 +597,7 @@ sort_gui_spec_popup_spec2d_bn_create(
       Xv_opaque	owner;
 #endif
 {
-#if NeedFunctionPrototype
-      extern void		spec2d_b1_nh(Panel_item, Event *);
-#else
-      extern void		spec2d_b1_nh();
-#endif
+      extern void spec2d_b1_nh P((Panel_item, Event *));
 	Xv_opaque	obj;
 	
 	obj = xv_create(owner, PANEL_BUTTON,
@@ -736,11 +624,7 @@ sort_gui_spec_popup_spec2d_b2_create(
       Xv_opaque	owner;
 #endif
 {
-#if NeedFunctionPrototype
-      extern void		spec2d_b2_nh(Panel_item, Event *);
-#else
-      extern void		spec2d_b2_nh();
-#endif
+      extern void spec2d_b2_nh P((Panel_item, Event *));
 	Xv_opaque	obj;
 	
 	obj = xv_create(owner, PANEL_BUTTON,
@@ -768,11 +652,7 @@ sort_gui_spec_popup_spec2d_b3_create(
       Xv_opaque	owner;
 #endif
 {
-#if NeedFunctionPrototype
-      extern void		spec2d_b3_nh(Panel_item, Event *);
-#else
-      extern void		spec2d_b3_nh();
-#endif
+      extern void spec2d_b3_nh P((Panel_item, Event *));
 	Xv_opaque	obj;
 	
 	obj = xv_create(owner, PANEL_BUTTON,
@@ -800,11 +680,7 @@ sort_gui_spec_popup_spec2d_b4_create(
       Xv_opaque	owner;
 #endif
 {
-#if NeedFunctionPrototype
-      extern void		spec2d_b4_nh(Panel_item, Event *);
-#else
-      extern void		spec2d_b4_nh();
-#endif
+      extern void spec2d_b4_nh P((Panel_item, Event *));
 	Xv_opaque	obj;
 	
 	obj = xv_create(owner, PANEL_BUTTON,
@@ -831,11 +707,7 @@ sort_gui_spec_popup_spec2d_b5_create(
       Xv_opaque	owner;
 #endif
 {
-#if NeedFunctionPrototype
-      extern void		spec2d_b5_nh(Panel_item, Event *);
-#else
-      extern void		spec2d_b5_nh();
-#endif
+      extern void spec2d_b5_nh P((Panel_item, Event *));
 	Xv_opaque	obj;
 	
 	obj = xv_create(owner, PANEL_BUTTON,
@@ -862,11 +734,7 @@ sort_gui_spec_popup_spec2d_b6_create(
       Xv_opaque	owner;
 #endif
 {
-#if NeedFunctionPrototype
-      extern void		spec2d_b6_nh(Panel_item, Event *);
-#else
-      extern void		spec2d_b6_nh();
-#endif
+      extern void spec2d_b6_nh P((Panel_item, Event *));
 	Xv_opaque	obj;
 	
 	obj = xv_create(owner, PANEL_BUTTON,
@@ -930,6 +798,7 @@ spec1d_list_nh(
 /*
  * Notify callback function for `spec1d_b1'.			"DISPLAY 1D"
  */
+/*ARGSUSED*/
 void
 spec1d_b1_nh(
 #if NeedFunctionPrototype
@@ -1016,6 +885,7 @@ spec1d_b1_nh(
 /*
  * Notify callback function for `spec1d_b3'.		"CLEAR ALL 1D"
  */
+/*ARGSUSED*/
 void
 spec1d_b3_nh(
 #if NeedFunctionPrototype
@@ -1050,6 +920,7 @@ spec1d_b3_nh(
 /*
  * Notify callback function for `spec1d_b4'.		"CLEAR SELECETED 1D"
  */
+/*ARGSUSED*/
 void
 spec1d_b4_nh(
 #if NeedFunctionPrototype
@@ -1112,10 +983,41 @@ spec1d_b4_nh(
 	return;
 }
 
+/***********************************************************************************************/
+/*
+ * Notify callback function for `spec1d_baction'.		Display action
+ */
+/*ARGSUSED*/
+void
+spec1d_baction_nh
+#if NeedFunctionPrototype
+(Panel_item	item,
+Event	*event)
+#else
+(item, event)
+Panel_item	item;
+Event		*event;
+#endif
+{
+    char *actname;
+
+    if ( DBX_val >= 6) fputs("sort_gui: spec1d_baction_nh\n", stderr);
+    
+    actname = (char *) xv_get(item, PANEL_CHOICE_STRING, xv_get(item, PANEL_VALUE));
+
+    sprintf(tmp_str,"displayaction %s", actname);
+
+    if (talk_to_server(TRUE,tmp_str,NULL) != 0)
+	return;
+    if ( DBX_val >= 2) fprintf(stderr,"sort_gui: display action set to %s\n", actname);
+
+}
+
 /***********************************************************************************************/
 /*
  * Notify callback function for `spec_clearall_but'.		"CLEAR ALL"
  */
+/*ARGSUSED*/
 void
 spec_clearall_nh(
 #if NeedFunctionPrototype
@@ -1196,6 +1098,7 @@ spec2d_list_nh(
 /*
  * Notify callback function for `spec2d_b1'.			"DISPLAY 2D"
  */
+/*ARGSUSED*/
 void
 spec2d_b1_nh(
 #if NeedFunctionPrototype
@@ -1255,6 +1158,7 @@ spec2d_b1_nh(
 /*
  * Notify callback function for `spec2d_b2'.		"CLEAR ALL 2D"
  */
+/*ARGSUSED*/
 void
 spec2d_b2_nh(
 #if NeedFunctionPrototype
@@ -1289,6 +1193,7 @@ spec2d_b2_nh(
 /*
  * Notify callback function for `spec2d_b3'.		"CLEAR SELECTED 2D"
  */
+/*ARGSUSED*/
 void
 spec2d_b3_nh(
 #if NeedFunctionPrototype
@@ -1345,6 +1250,7 @@ spec2d_b3_nh(
 /*
  * Notify callback function for `spec2d_b4'.		"SHOW WINDOWS POPUP"
  */
+/*ARGSUSED*/
 void
 spec2d_b4_nh(
 #if NeedFunctionPrototype
@@ -1365,6 +1271,7 @@ spec2d_b4_nh(
 /*
  * Notify callback function for `spec2d_b5'.		"RESET GRAPHICS"
  */
+/*ARGSUSED*/
 void
 spec2d_b5_nh(
 #if NeedFunctionPrototype
@@ -1386,6 +1293,7 @@ spec2d_b5_nh(
 /*
  * Notify callback function for `spec2d_b5'.		"REFRESH GUI"
  */
+/*ARGSUSED*/
 void
 spec2d_b6_nh(
 #if NeedFunctionPrototype
