@@ -6,167 +6,46 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <memory.h>
 #include <stdlib.h>
-#include <alloca.h>
-#include <errno.h>
 
 #include <sys/stat.h>
-#include <sys/param.h>
 #include <sys/types.h>
 
 #include <xview/xview.h>
 #include <xview/canvas.h>
-#include <xview/icon_load.h>
 #include <xview/panel.h>
-#include <xview/scrollbar.h>
 #include <xview/svrimage.h>
-#include <xview/termsw.h>
-#include <xview/text.h>
-#include <xview/tty.h>
-#include <xview/xv_xrect.h>
 
+#include "sort_def.h"
+#include "sort_thread.h"
 #include "gui_extern.h"
-#include "gui.h"
+#include "gui_alerts.h"
 #include "version.h"
 #include "../sunsort_lib/formats.h"
+#include "gui_main.h"
+#include "gui_ui_main.h"
+#include "gui_toolpop.h"
 
-extern Xv_opaque
-sort_gui_tool_popup_create(
-#if NeedFunctionPrototype
-				Xv_opaque
-#endif
-				);
-
-extern Xv_opaque
-sort_gui_tool_popup_control_create(
-#if NeedFunctionPrototype
-
-				     Xv_opaque		     
-#endif
-				     );
-
-extern Xv_opaque
-sort_gui_tool_popup_tf1_create(
-#if NeedFunctionPrototype
-				 Xv_opaque
-#endif
-				 );
-
-extern Xv_opaque
-sort_gui_tool_popup_tf2_create(
-#if NeedFunctionPrototype
-				 Xv_opaque
-#endif
-				 );
-
-extern Xv_opaque
-sort_gui_tool_popup_tf3_create(
-#if NeedFunctionPrototype
-				 Xv_opaque
-#endif
-				 );
-
-extern Xv_opaque
-sort_gui_tool_popup_tf4_create(
-#if NeedFunctionPrototype
-				 Xv_opaque
-#endif
-				 );
-
-extern Xv_opaque
-sort_gui_tool_popup_tf5_create(
-#if NeedFunctionPrototype
-				 Xv_opaque
-#endif
-				 );
-extern Xv_opaque
-sort_gui_tool_popup_b1_create(
-#if NeedFunctionPrototype
-				 Xv_opaque
-#endif
-				 );
-extern Xv_opaque
-sort_gui_tool_popup_b2_create(
-#if NeedFunctionPrototype
-				   Xv_opaque
-#endif
-				   );
-
-extern Xv_opaque
-sort_gui_tool_popup_b3_create(
-#if NeedFunctionPrototype
-				   Xv_opaque
-#endif
-				   );
-
-extern Xv_opaque
-sort_gui_tool_popup_b4_create(
-#if NeedFunctionPrototype
-				   Xv_opaque
-#endif
-				   );
-
-extern Xv_opaque
-sort_gui_tool_popup_b5_create(
-#if NeedFunctionPrototype
-				   Xv_opaque
-#endif
-				   );
-
-extern Xv_opaque
-sort_gui_tool_popup_b6_create(
-#if NeedFunctionPrototype
-				   Xv_opaque
-#endif
-				   );
-
-extern Xv_opaque
-sort_gui_tool_popup_setting1_create(
-#if NeedFunctionPrototype
-					 Xv_opaque
-#endif
-					 );
-
-extern Xv_opaque
-sort_gui_tool_popup_setting2_create(
-#if NeedFunctionPrototype
-					 Xv_opaque
-#endif
-					 );
-
-extern Xv_opaque
-sort_gui_tool_popup_setting3_create(
-#if NeedFunctionPrototype
-					 Xv_opaque
-#endif
-					 );
-
-extern Xv_opaque
-sort_gui_tool_popup_setting4_create(
-#if NeedFunctionPrototype
-					 Xv_opaque
-#endif
-					 );
-extern Xv_opaque
-sort_gui_tool_popup_setting5_create(
-#if NeedFunctionPrototype
-					 Xv_opaque
-#endif
-					 );
-extern Xv_opaque
-sort_gui_tool_popup_setting6_create(
-#if NeedFunctionPrototype
-					 Xv_opaque
-#endif
-					 );
-
-extern Xv_opaque
-sort_gui_tool_popup_setting7_create(
-#if NeedFunctionPrototype
-					 Xv_opaque
-#endif
-					 );
+Xv_opaque sort_gui_tool_popup_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_control_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_tf1_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_tf2_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_tf3_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_tf4_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_tf5_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_b1_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_b2_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_b3_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_b4_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_b5_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_b6_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_setting1_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_setting2_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_setting3_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_setting4_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_setting5_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_setting6_create P((Xv_opaque));
+Xv_opaque sort_gui_tool_popup_setting7_create P((Xv_opaque));
 
 /*
  * Initialize an instance of object `tool_popup'.
@@ -273,8 +152,8 @@ sort_gui_tool_popup_create(
 		XV_HEIGHT, 64,
 		NULL);
 	obj = xv_create(owner, FRAME,
-		XV_WIDTH, 413,
-		XV_HEIGHT, 390,
+		XV_WIDTH, 412,
+		XV_HEIGHT, 412,
 		XV_LABEL, "SunSort tool",
 		XV_SHOW, TRUE,
 		FRAME_SHOW_FOOTER, TRUE,
@@ -356,12 +235,7 @@ sort_gui_tool_popup_b1_create(
 #endif
 {
 	Xv_opaque	obj;
-	
-#if NeedFunctionPrototype
-      extern void		tool_b1_nh(Panel_item, Event *); 
-#else
-      extern void		tool_b1_nh();
-#endif
+      extern void tool_b1_nh P((Panel_item, Event *);) 
 	
 	obj = xv_create(owner, PANEL_BUTTON,
 		XV_X, 340,
@@ -386,12 +260,7 @@ sort_gui_tool_popup_setting1_create(
 #endif
 {
 	Xv_opaque	obj;
-	
-#if NeedFunctionPrototype
-      extern int		tool_setting1_nh(Panel_item, int, Event *); 
-#else
-      extern int		tool_setting1_nh();
-#endif
+      extern void tool_setting1_nh P((Panel_item, int, Event *);) 
 	
 	obj = xv_create(owner, PANEL_CHOICE,
 		XV_X, 170,
@@ -482,11 +351,8 @@ sort_gui_tool_popup_b2_create(
 #endif
 {
 	Xv_opaque	obj;
-#if NeedFunctionPrototype
-      extern void		tool_b2_nh(Panel_item, Event *); 
-#else
-      extern void		tool_b2_nh();
-#endif
+      extern void tool_b2_nh P((Panel_item, Event *);) 
+
 	obj = xv_create(owner, PANEL_BUTTON,
 		XV_X, 20,
 		XV_Y, 180,
@@ -510,11 +376,8 @@ sort_gui_tool_popup_b3_create(
 #endif
 {
 	Xv_opaque	obj;
-#if NeedFunctionPrototype
-      extern void		tool_b3_nh(Panel_item, Event *); 
-#else
-      extern void		tool_b3_nh();
-#endif	
+      extern void tool_b3_nh P((Panel_item, Event *);) 
+
 	obj = xv_create(owner, PANEL_BUTTON,
 		XV_X, 90,
 		XV_Y, 180,
@@ -538,11 +401,8 @@ sort_gui_tool_popup_b4_create(
 #endif
 {
 	Xv_opaque	obj;
-#if NeedFunctionPrototype
-      extern void		tool_b4_nh(Panel_item, Event *); 
-#else
-      extern void		tool_b4_nh();
-#endif	
+      extern void tool_b4_nh P((Panel_item, Event *);) 
+
 	obj = xv_create(owner, PANEL_BUTTON,
 		XV_X, 160,
 		XV_Y, 180,
@@ -566,12 +426,7 @@ sort_gui_tool_popup_setting2_create(
 #endif
 {
 	Xv_opaque	obj;
-	
-#if NeedFunctionPrototype
-      extern int		tool_setting2_nh(Panel_item, int, Event *); 
-#else
-      extern int		tool_setting2_nh();
-#endif
+      extern void tool_setting2_nh P((Panel_item, int, Event *);) 
 
 	obj = xv_create(owner, PANEL_TOGGLE,
 		XV_X, 25,
@@ -602,17 +457,12 @@ sort_gui_tool_popup_setting3_create(
 #endif
 {
 	Xv_opaque	obj;
-	
-#if NeedFunctionPrototype
-      extern int		tool_setting3_nh(Panel_item, int, Event *); 
-#else
-      extern int		tool_setting3_nh();
-#endif
+      extern void tool_setting3_nh P((Panel_item, int, Event *);) 
 
 	obj = xv_create(owner, PANEL_CHOICE,
 		XV_X, 25,
 		XV_Y, 330,
-		PANEL_CHOICE_NROWS, 2,
+		PANEL_CHOICE_NROWS, 3,
 		PANEL_LAYOUT, PANEL_HORIZONTAL,
 		PANEL_CHOOSE_NONE, TRUE,
 	        PANEL_VALUE, 0x0,
@@ -639,12 +489,7 @@ sort_gui_tool_popup_setting4_create(
 #endif
 {
 	Xv_opaque	obj;
-	
-#if NeedFunctionPrototype
-      extern int		tool_setting4_nh(Panel_item, int, Event *); 
-#else
-      extern int		tool_setting4_nh();
-#endif
+      extern void tool_setting4_nh P((Panel_item, int, Event *);) 
 
 	obj = xv_create(owner, PANEL_TOGGLE,
 		XV_X, 25,
@@ -674,11 +519,7 @@ sort_gui_tool_popup_setting5_create(
 {
 	Xv_opaque	obj;
 	
-#if NeedFunctionPrototype
-      extern int		tool_setting5_nh(Panel_item, int, Event *); 
-#else
-      extern int		tool_setting5_nh();
-#endif
+      extern void tool_setting5_nh P((Panel_item, int, Event *);) 
 
 	obj = xv_create(owner, PANEL_TOGGLE,
 		XV_X, 85,
@@ -708,12 +549,7 @@ sort_gui_tool_popup_setting6_create(
 #endif
 {
 	Xv_opaque	obj;
-	
-#if NeedFunctionPrototype
-      extern int		tool_setting6_nh(Panel_item, int, Event *); 
-#else
-      extern int		tool_setting6_nh();
-#endif
+      extern void tool_setting6_nh P((Panel_item, int, Event *);) 
 
 	obj = xv_create(owner, PANEL_TOGGLE,
 		XV_X, 85,
@@ -742,12 +578,7 @@ sort_gui_tool_popup_setting7_create(
 #endif
 {
 	Xv_opaque	obj;
-	
-#if NeedFunctionPrototype
-      extern int		tool_setting7_nh(Panel_item, int, Event *); 
-#else
-      extern int		tool_setting7_nh();
-#endif
+      extern void tool_setting7_nh P((Panel_item, int, Event *);) 
 
 	obj = xv_create(owner, PANEL_TOGGLE,
 		XV_X, 160,
@@ -831,12 +662,8 @@ sort_gui_tool_popup_b5_create(
 #endif
 {
 	Xv_opaque	obj;
-	
-#if NeedFunctionPrototype
-      extern void		tool_b5_nh(Panel_item, Event *); 
-#else
-      extern void		tool_b5_nh();
-#endif
+      extern void tool_b5_nh P((Panel_item, Event *);) 
+
 	obj = xv_create(owner, PANEL_BUTTON,
 		XV_X, 230,
 		XV_Y, 180,
@@ -860,12 +687,7 @@ sort_gui_tool_popup_b6_create(
 #endif
 {
 	Xv_opaque	obj;
-	
-#if NeedFunctionPrototype
-      extern void		tool_b6_nh(Panel_item, Event *); 
-#else
-      extern void		tool_b6_nh();
-#endif
+      extern void tool_b6_nh P((Panel_item, Event *);) 
 	
 	obj = xv_create(owner, PANEL_BUTTON,
 		XV_X, 340,
@@ -883,6 +705,7 @@ sort_gui_tool_popup_b6_create(
 /*
  * Notify callback function for `tool_b1'.  "EDIT SORT"
  */
+/*ARGSUSED*/
 void
 tool_b1_nh(
 #if NeedFunctionPrototype
@@ -943,6 +766,7 @@ tool_b1_nh(
 /*
  * Notify callback function for `tool_b6'.  "EDIT SPEC"
  */
+/*ARGSUSED*/
 void
 tool_b6_nh(
 #if NeedFunctionPrototype
@@ -1003,6 +827,7 @@ tool_b6_nh(
 /*
  * Notify callback function for `tool_b2'.  "MAKESORT"
  */
+/*ARGSUSED*/
 void
 tool_b2_nh
 #if NeedFunctionPrototype
@@ -1018,8 +843,6 @@ Event	*event)
     char  *specfile;
     char  *sort_proc;
     char  *other_opt_string;
-    char  *spec_string;
-    char  *c_flag;
     int   debug_opt;
     int   csort_opt;
     int   spec_opt;
@@ -1135,6 +958,7 @@ Event	*event)
 /*
  * Notify callback function for `tool_b3'.  "LOAD SORT"
  */
+/*ARGSUSED*/
 void
 tool_b3_nh(
 #if NeedFunctionPrototype
@@ -1180,6 +1004,7 @@ Do you wish to reload the spectra as well?";
 /*
  * Notify callback function for `tool_b4'.  "DEBUG SORT"
  */
+/*ARGSUSED*/
 void
 tool_b4_nh(
 #if NeedFunctionPrototype
@@ -1212,6 +1037,7 @@ tool_b4_nh(
 /*
  * Notify callback function for `tool_b5'.    "KILL SORT"
  */
+/*ARGSUSED*/
 void
 tool_b5_nh(
 #if NeedFunctionPrototype
@@ -1244,7 +1070,8 @@ tool_b5_nh(
 /*
  * Notify callback function for `tool_setting1'.  
  */
-int
+/*ARGSUSED*/
+void
 tool_setting1_nh(
 #if NeedFunctionPrototype
 	     Panel_item	item,
@@ -1264,14 +1091,13 @@ tool_setting1_nh(
 	      xv_set(sort_gui_tool_popup->tool_tf2, XV_SHOW, FALSE, NULL);
 	else if (value == 1)
 	      xv_set(sort_gui_tool_popup->tool_tf2, XV_SHOW, TRUE, NULL);
-
-	return;
 }
 /*********************************************************************************************/
 /*
  * Notify callback function for `tool_setting2'.  
  */
-int
+/*ARGSUSED*/
+void
 tool_setting2_nh(
 #if NeedFunctionPrototype
 	     Panel_item	item,
@@ -1286,14 +1112,13 @@ tool_setting2_nh(
 {
 	if ( DBX_val >= 6)
 	      fprintf(stderr,"sort_gui: tool_setting2_nh value %d\n", value);
-         
-	return;
 }
 /*********************************************************************************************/
 /*
  * Notify callback function for `tool_setting3'.  
  */
-int
+/*ARGSUSED*/
+void
 tool_setting3_nh(
 #if NeedFunctionPrototype
 	     Panel_item	item,
@@ -1315,19 +1140,16 @@ tool_setting3_nh(
 	    format = (char *) xv_get(item, PANEL_CHOICE_STRING, value);
 	    
 	    /* communicate with sort server */
-	    if (talk_to_server(TRUE,"setsenv format ", format, NULL) != 0) {
+	    if (talk_to_server(TRUE,"setsenv format ", format, NULL) != 0)
 		  xv_set(item, PANEL_VALUE, -1, NULL);
-		  return;
-	    }
       }
-
-     return;
 }
 /*********************************************************************************************/
 /*
  * Notify callback function for `tool_setting4'.  
  */
-int
+/*ARGSUSED*/
+void
 tool_setting4_nh(
 #if NeedFunctionPrototype
 	     Panel_item	item,
@@ -1342,14 +1164,13 @@ tool_setting4_nh(
 {
 	if ( DBX_val >= 6)
 	      fprintf(stderr,"sort_gui: tool_setting4_nh value %d\n", value);
-         
-	return;
 }
 /*********************************************************************************************/
 /*
  * Notify callback function for `tool_setting5'.  
  */
-int
+/*ARGSUSED*/
+void
 tool_setting5_nh(
 #if NeedFunctionPrototype
 	     Panel_item	item,
@@ -1373,14 +1194,13 @@ tool_setting5_nh(
 	      xv_set(sort_gui_tool_popup->tool_tf5, XV_SHOW, TRUE, NULL);
 	      xv_set(sort_gui_tool_popup->tool_b6, XV_SHOW, TRUE, NULL);
         }
-         
-	return;
 }
 /*********************************************************************************************/
 /*
  * Notify callback function for `tool_setting6'.  
  */
-int
+/*ARGSUSED*/
+void
 tool_setting6_nh(
 #if NeedFunctionPrototype
 	     Panel_item	item,
@@ -1400,14 +1220,12 @@ tool_setting6_nh(
 	      xv_set(sort_gui_tool_popup->tool_tf4, XV_SHOW, FALSE, NULL);
 	else if (value == 1)
 	      xv_set(sort_gui_tool_popup->tool_tf4, XV_SHOW, TRUE, NULL);
-
-         
-	return;
 }
 /*
  * Notify callback function for `tool_setting7'.  
  */
-int
+/*ARGSUSED*/
+void
 tool_setting7_nh(
 #if NeedFunctionPrototype
 	     Panel_item	item,
@@ -1422,7 +1240,5 @@ tool_setting7_nh(
 {
 	if ( DBX_val >= 6)
 	      fprintf(stderr,"sort_gui: tool_setting7_nh value %d\n", value);
-         
-	return;
 }
 /*********************************************************************************************/
