@@ -3,9 +3,7 @@
  * Most of the functionality is in rdtape_eg.c
  */
 
-#include <stdio.h>
-
-#include "sort_thread.h"
+#include "sort_def.h"
 #include "rdtape.h"
 #include "rdtape_eg.h"
 
@@ -18,7 +16,7 @@ rdtape_demon(
       int filter;
 #endif
 {
-    return(rdtape_eg(filter, STYLE_EGAM));
+    return rdtape_eg(filter, STYLE_EGAM);
 }
 
 int
