@@ -17,7 +17,6 @@
 
 
 #include <stdio.h>
-#include <memory.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -26,7 +25,7 @@
 #include <sys/types.h>
 #include <sys/time.h>
 
-#include "sort_thread.h"
+#include "sort_def.h"
 #include "rdtape.h"
 #include "rdtape_eg.h"
 
@@ -44,6 +43,10 @@
 #define EG_TAPE_3          0x4144
 #define EG_TAPE_4          0x4154
 
+#define MEGHA_MAX_DUMMY	   2 /* Maximum number of extra dummy cycles allowed
+				in Megha format data */
+#define ALLOW_EVENT_STAMPING /* Define this for debugging with Starburst
+				event stamping */
 /*
 #define DUMP_RECORDS
 #define PRINT_BLOCKS
@@ -72,13 +75,13 @@ static short *addr2adc[EG_ADDR_SIZE];
 static short adc2addr[NOS_ADC_WORDS];
 
 int
-rdtape_eg(
+rdtape_eg
 #if NeedFunctionPrototype
-	  int filter, int style)
+(int filter, int style)
 #else
-      filter, style)
-      int filter;
-      int style;
+(filter, style)
+int filter;
+int style;
 #endif
 {
     register short *ebye;                     /* pointer to input buffer */
@@ -86,6 +89,7 @@ rdtape_eg(
     
     register int *eptr, *header_ptr, *event_end;
     
+    short *ap, *adclist;
     time_t start_time, time_now;
     int bytes, record = 0;
     int adc_count = 0;
@@ -93,11 +97,16 @@ rdtape_eg(
     int bad, badlong = 0, badrecords = 0, recovered = 0, badfirst = 0;
     int badlength = 0, badword = 0, badshort = 0, recoverlong = 0;
     int recovershort = 0, total, badeob = 0, badtrunc = 0, bad16 = 0;
-    
+    int dummies[MEGHA_MAX_DUMMY + 2], dummy, overrun = 0, badfirstlength = 0;
+    int recoverfirstlength = 0, badcount = 0, badrepeatlength = 0;
+    int badoverrun = 0, badoverrun2 = 0, badstart = 0, recoveroverrun = 0;
+
     register int group, hwords;
     register struct eg_word_struct *wptr;
     register int adc_bytes;
-    int *end_ptr;
+    int *end_ptr, i;
+    unsigned short *ep;
+    unsigned int *ip;
 
 #ifdef DUMP_RECORDS
     if (dump_state == 2)
@@ -116,71 +125,161 @@ rdtape_eg(
     (void) memset((char *) adcs_.bitpattern, 0, sizeof(adcs_.bitpattern));
     adc_bytes = adcnum_.nos_adcs * sizeof(short);
       
-    /* call user's initialization function */      
-    (void) init_();
+    memset((char *) dummies, 0, sizeof(dummies));
+
+    /* call user's initialization function */ 
+    if (init()) {
+	fprintf(stderr, initfailmess);
+	return filter;
+    }
       
     /* set start time */
     start_time = time(NULL);
       
-    /* sort requested number of records or until end of file etc */
+    /* sort requested number of records or until end of file or error etc */
     for (record=1; ;record++) {
-	if ( (ebye = read_block(&bytes)) == NULL) 
+	if ((ebye = read_block(&bytes)) == NULL) 
 	    break;
-
+	    
+	/*LINTED*/
 	end_ptr = ((int *) ebye) + (bytes>>2);
-
+	    
 #ifdef PRINT_BLOCKS
 	fprintf(stderr, "Block starts as follows:\n");
-    {
-	int i;
-	for(i=0; i<256; i++)
 	{
-	    fprintf(stderr, "%04x", ebye[i] & 0xffff);
-	    if ((i & 15) == 15)
-		putc('\n', stderr);
-	    else
-		putc(' ', stderr);
-	}
-    }
-#endif
-	/* check start of record header to see if it a tape block*/
-	if (memcmp("MEGHADAT", (char *) ebye, 8) == 0 ||
-	    memcmp("EGEVENTD", (char *) ebye, 8) == 0) {
-	    if ( ebye[12] != (short) DELIMITER_MASK) {
-		fprintf(stderr,"First event header is not a start token "
-			"... trying next record\n");
-		badfirst++;
-#ifdef DUMP_RECORDS
-		bad = 1;
-		goto EVENT_LOOP_END;
-#else
-		continue;
-#endif
-	    }
-	    eptr = (int *) &ebye[12];
-	}
-	/* if not a tape block then just treat as before */
-	else {
-	    eptr = (int *) &ebye[0];
-	    if ( ebye[0] != (short) DELIMITER_MASK) {
-		fprintf(stderr,"First event header is not a start token "
-			"... trying next record. ");
-		fprintf(stderr,"First 6 words were (HSM order): "
-			"%04x %04x %04x %04x %04x %04x\n",
-			ebye[1] & 0xffff, ebye[0] & 0xffff, ebye[3] & 0xffff,
-			ebye[2] & 0xffff, ebye[5] & 0xffff, ebye[4] & 0xffff);
-		badfirst++;
-#ifdef DUMP_RECORDS
-		bad = 1;
-		goto EVENT_LOOP_END;
-#else
-		continue;
-#endif
+	    for(i=0; i<256; i++)
+	    {
+		fprintf(stderr, "%04x", ebye[i] & 0xffff);
+		if ((i & 15) == 15)
+		    putc('\n', stderr);
+		else
+		    putc(' ', stderr);
 	    }
 	}
-	    
+#endif
+
 	bad = 0;
 
+	/* check start of record header to see if it is a tape block*/
+ 	if (memcmp("MEGHADAT", (char *) ebye, 8) == 0 ||
+ 	    memcmp("EGEVENTD", (char *) ebye, 8) == 0)
+	    /*LINTED*/
+	    eptr = (int *) &ebye[12];
+	else
+	    /*LINTED*/
+	    eptr = (int *) &ebye[0];
+
+	if ((*eptr & END_TOKEN) != END_TOKEN) {
+	    switch(style)
+	    {
+	    case STYLE_MEGHA:
+		ep = (unsigned short *) eptr;
+
+		if (ep[1] == 0xffff)
+		{
+		    fprintf(stderr, "Length missing on first event. "
+			    "Skipping event. ");
+		    end_ptr--;
+		    for(ip = (unsigned int *) eptr;
+			ip < (unsigned int *) end_ptr; ip++)
+			*ip = (*ip>>16) | (ip[1]<<16);
+		    badfirstlength++;
+		    bad = 1;
+		    while(eptr < end_ptr &&(*eptr & END_TOKEN) != END_TOKEN)
+			eptr++;
+		    if (eptr == end_ptr)
+		    {
+			fprintf(stderr, "Unable to recover, skipping "
+				"record.\n");
+			overrun = 0;
+			goto EVENT_LOOP_END;
+		    }
+		    fprintf(stderr, "Recovering at next event.\n");
+		    recoverfirstlength++;
+		    break;
+		}
+		if (ep[3] == 0xffff && ep[0] != 0 && ep[1] != 0
+#ifndef ALLOW_EVENT_STAMPING
+		    && (ep[0] & 0xf000) == 0 && (ep[1] & 0xf000) == 0
+#endif
+		    )
+		{
+		    fprintf(stderr, "Repeated event length on first event. "
+			    "Skipping event.\n");
+		    end_ptr--;
+		    for(ip = (unsigned int *) eptr;
+			ip < (unsigned int *) end_ptr; ip++)
+			*ip = (*ip>>16) | (ip[1]<<16);
+		    badrepeatlength++;
+		    bad = 1;
+		    break;
+		}
+		for(; ep < (unsigned short *) end_ptr && *ep != 0xffff; ep++)
+		    if ((*ep & 0xe000) != 0x2000 && (*ep & 0xe000) != 0x8000
+			&& *ep != 0 &&
+			(ep + 3 >= (unsigned short *) end_ptr ||
+			 ep[3] != 0xffff))
+			break;
+		if (ep < (unsigned short *) end_ptr && *ep == 0xffff)
+		{
+		    if (((((char *) ep)-((char*) eptr)) & (sizeof(int)-1))
+			!= 0)
+		    {
+			ep -= 3;
+			/*LINTED*/
+			for(ip = (unsigned int *) ep;
+			    ip < (unsigned int *) end_ptr; ip++)
+			    *ip = (*ip>>16) | (ip[1]<<16);
+		    }
+		    /*LINTED*/
+		    eptr = (int *) ep;
+		    if (*eptr == END_TOKEN)
+		    {
+			if (overrun)
+			{
+			    fprintf(stderr, "Data overrun from previous "
+				    "record. Skipping record.\n");
+			    badoverrun++;
+			    bad = 1;
+			    overrun = 0;
+			    goto EVENT_LOOP_END;
+			}
+			fprintf(stderr, "Data overrun from unknown source."
+				" Skipping record.\n");
+			badoverrun2++;
+			bad = 1;
+			overrun = 0;
+			goto EVENT_LOOP_END;
+		    }
+		    if (overrun)
+		    {
+			fprintf(stderr, "Big data overrun from previous "
+				"record. Recovering some events. \n");
+			badoverrun++;
+			bad = 1;
+			recoveroverrun++;
+			break;
+		    }
+		    fprintf(stderr, "Missing start of event. Recovering at "
+			    "next event.\n");
+		    badstart++;
+		    bad = 1;
+		    break;
+		}
+		/* FALL THROUGH */
+	    default:
+		fprintf(stderr,"First event header is not a start token ... "
+			"trying next record.\nFirst 8 words were (HSM order): "
+			"%04x %04x %04x %04x %04x %04x %04x %04x\n", ep[1],
+			ep[0], ep[3], ep[2], ep[5], ep[4], ep[7], ep[6]);
+		badfirst++;
+		bad = 1;
+		goto EVENT_LOOP_END;
+	    }
+	}
+	
+	overrun = 0;
+	
 	/* search for end of block indication (header == END_TOKEN) */
 	while( *eptr != (int) END_TOKEN) {
 	    /* extract fields from event header */
@@ -188,42 +287,45 @@ rdtape_eg(
 	    switch(style)
 	    {
 	    case STYLE_MEGHA:
+#ifdef ALLOW_EVENT_STAMPING
+		event_len = (*header_ptr & 0x00000fff) >> 1;
+#else
 		event_len = (*header_ptr & 0x0000ffff) >> 1;
+#endif
+		dummy = 0;
 		break;
 	    default:
 		event_len = (*header_ptr & 0x0000ffff) >> 2;
 		break;
 	    }
-
+	    
 	    /* check that event length is sensible */
 	    if((event_len < 0) || (event_len > 4096)) {
 		fprintf(stderr,"Event length (%d  %04x) out of range ... "
-			"skipping  rest of record %d\n", event_len,
+			"skipping rest of record %d\n", event_len,
 			event_len, record);
 		badlength++;
-#ifdef DUMP_RECORDS
 		bad = 1;
-#endif
 		break;
 	    }
 	    event_end = header_ptr + event_len;
-
+	    
 	    if (event_end >= end_ptr)
 	    {
 		fprintf(stderr, "Last event in block truncated - "
 			"ignoring it.\n");
 		badtrunc++;
-#ifdef DUMP_RECORDS
+		overrun = 1;
 		bad = 1;
-#endif
 		break;
 	    }
 	    
 	    /* reset adcs_common block */
 	    (void) memset((char *) adcs_.adcs, -1, adc_bytes);
-            (void) memset((char *) groups, 0, sizeof(groups));
+	    (void) memset((char *) groups, 0, sizeof(groups));
 	    adc_count = 0;
-		  
+	    adclist = adclist_.adclist;
+	    
 	    /* unpack individual event ** EVENT LOOP ** */		  
 	    while ( (*eptr & (int) END_TOKEN) != (int) END_TOKEN)  {
 		if (eptr >= event_end) {
@@ -235,8 +337,8 @@ rdtape_eg(
 			    (*eptr & (int) END_TOKEN) != (int) END_TOKEN);
 		    if ((*eptr & (int) END_TOKEN) != (int) END_TOKEN)
 		    {
-			int *ep, i;
-
+			int *ep;
+			
 			fprintf(stderr, "End of block at short word %d "
 				"without end of block marker in record %d\n",
 				((short *) sptr) - ebye, record);
@@ -255,6 +357,7 @@ rdtape_eg(
 			}
 			fprintf(stderr, "\n");
 			badeob++;
+			bad = 1;
 			goto EVENT_LOOP_END;
 		    }
 		    if (!bad)
@@ -266,51 +369,64 @@ rdtape_eg(
 		    if ((eptr-header_ptr)*4 > 1024)
 		    {
 			fprintf(stderr, "Expected event length %d, no start "
-                                "of event marker within 1024 (next at %d). "
-                                "Not recovering.\n");
-				goto EVENT_LOOP_END;
+				"of event marker within 1024 (next at %d). "
+				"Not recovering.\n",
+				event_len*4, (eptr-header_ptr)*4);
+			goto EVENT_LOOP_END;
 		    }
 		    fprintf(stderr, "Expected event length %d, actually %d, "
-			    "error %d. Recovery succeeded.\n", event_len*4,
-			    (eptr-header_ptr)*4,
+			    "error %d. Recovering at next event.\n",
+			    event_len*4, (eptr-header_ptr)*4,
 			    (eptr-header_ptr-event_len)*4);
 		    recoverlong++;
 		    goto EVENT_LOOP_CONTINUE;
 		}
 		
-		/* event type defines tells us how much data is to follow */
+		/* event type defines tells us how much data is to follow*/
 		wptr = (struct eg_word_struct *) eptr++;
 		switch(style)
 		{
 		case STYLE_MEGHA:
 		    switch ((*(eptr-1)) >> 29) {
 		    case 1:           /* simple parameter word found */
-			if (addr2adc[wptr->addr & 0x1fff])
-			    *addr2adc[wptr->addr & 0x1fff] =
-				wptr->data & 0x1fff;
+			/*LINTED*/
+			if (ap = addr2adc[wptr->addr & 0x1fff])
+			{
+			    *ap = wptr->data & 0x1fff;
+			    *adclist++ = ap - adcs_.adcs + 1;
+			}
 			adc_count++;
 			break;
 		    case 2:           /* group format parameter(s) found */
 			group = wptr->addr & 0x00ff;
 			hwords = (wptr->addr >> 8) & 0x003f; 
-                        groups[group].group_len = hwords;
-                        groups[group].group_ptr = &wptr->data;
+			groups[group].group_len = hwords;
+			groups[group].group_ptr = &wptr->data;
 			eptr += (hwords / 2);
 			break;
+		    case 0:	       /* possible dummy cycle */
+			if (*(eptr-1) == 0 || *(eptr-1) == 0x02000000)
+			{
+			    dummy++;
+			    break;
+			}
+			/* FALL THROUGH */
 		    default:           /* invalid word format found  */
 			if ((*eptr & 0xffff) == 0xffff ||
 			    (*eptr & 0xe000) == 0x2000)
 			{
-			    fprintf(stderr, "16 bit data skew in HSM (urk).\n");
+			    fprintf(stderr, "16 bit data skew in HSM "
+				    "(urk).\n");
 			    bad16++;
 			}
 			else
 			{    
-			    int *ep, i;
+			    int *ep;
 			    
-			    fprintf(stderr,"invalid word format found ... "
-				    "skipping rest of record %d\n", record);
-			    fprintf(stderr,"Words surrounding error were:\n    ");
+			    fprintf(stderr,"Invalid word format found ... "
+				    "skipping rest of record %d\n",record);
+			    fprintf(stderr,"Words surrounding error were:"
+				    "\n    ");
 			    for(ep = eptr - 10, i = 4;
 				ep < eptr+30 && ep < end_ptr; ep++)
 			    {
@@ -326,17 +442,19 @@ rdtape_eg(
 			    fprintf(stderr, "\n");
 			    badword++;
 			}		    
-#ifdef DUMP_RECORDS
 			bad = 1;
-#endif
 			goto EVENT_LOOP_END;
 		    }
 		    break;
 		default:
 		    switch (wptr->type) {
 		    case 0:           /* simple parameter word found */
-			if (addr2adc[wptr->addr])
-			    *addr2adc[wptr->addr] = wptr->data;
+			/*LINTED*/
+			if (ap = addr2adc[wptr->addr])
+			{
+			    *ap = wptr->data;
+			    *adclist++ = ap - adcs_.adcs + 1;
+			}
 			adc_count++;
 			break;
 		    case 1:           /* group format parameter(s) found */
@@ -348,7 +466,7 @@ rdtape_eg(
 			break;
 		    default:           /* invalid word format found  */
 		    {
-			int *ep, i;
+			int *ep;
 			
 			fprintf(stderr,"invalid word format found ... "
 				"skipping rest of record %d\n", record);
@@ -366,12 +484,10 @@ rdtape_eg(
 			    i += 9;
 			}
 		    }
-			fprintf(stderr, "\n");
-			badword++;
-#ifdef DUMP_RECORDS
-			bad = 1;
-#endif
-			goto EVENT_LOOP_END;
+		    fprintf(stderr, "\n");
+		    badword++;
+		    bad = 1;
+		    goto EVENT_LOOP_END;
 		    }
 		    break;
 		}
@@ -388,22 +504,40 @@ rdtape_eg(
 		badshort++;
 		recovershort++;
 		fprintf(stderr, "Expected event length %d, actually %d, "
-			"error %d. Recovery succeeded.\n", event_len*2,
-			(eptr-header_ptr)*2, (eptr-header_ptr-event_len)*2);
+			"error %d. Recovering at next event.\n",
+			event_len*2, (eptr-header_ptr)*2,
+			(eptr-header_ptr-event_len)*2);
 		continue;
 	    }
 	    
+	    if (style == STYLE_MEGHA)
+	    {
+		if (dummy > MEGHA_MAX_DUMMY)
+		{
+		    fprintf(stderr,
+			    "Too many dummy cycles, skipping event.\n");
+		    dummies[MEGHA_MAX_DUMMY+1]++;
+		    continue;
+		}
+		else
+		    dummies[dummy]++;
+	    }
+
 	    /* provide access to various information to user routine */
 	    adcs_.triggernos = adc_count;
 	    adcs_.event = ++in_events;
 	    adcs_.record = record;
-            adcs_.wrtevt = 0;
+	    adcs_.wrtevt = 0;
+	    
+	    *adclist = -1;
 	    if (bad)
 		recovered++;
 	    
 	    /* call user provided event sorting routine */
-	    (void) sortin_();
-	    
+	    if (sortin()) {
+		fprintf(stderr, sortinfailmess);
+		goto ABORT_SORT;
+	    }	    
 	    /* write out event if wrtevt set to true */
 	    if (filter && adcs_.wrtevt) {
 		if ( write_filt((short *)header_ptr, 2*event_len) )
@@ -413,10 +547,13 @@ rdtape_eg(
 	    ;
 	    
 	}    /* loop on ebye buffer until end token */
-	    
+	
 	/* come to here to jump out of faulty record */		  
     EVENT_LOOP_END:
 	bytes = 0;
+	
+	if (bad)
+	    badcount++;
 
 #ifdef DUMP_RECORDS
 	if (dump_state == 1
@@ -427,8 +564,7 @@ rdtape_eg(
 	{
 	    if (!--dump_nth_count)
 	    {
-		int i;
-
+		
 		write(dump_fd, ebye, 16384);
 		if (!--dump_record_count)
 		{
@@ -443,12 +579,15 @@ rdtape_eg(
 	
     }   /* loop over requested number of records or until eof */
     
+ABORT_SORT:
+    
     /* lets note the time at end */
     time_now = time(NULL);
     
     /* call user provided clearup routine  */
-    (void) finish_();
-      
+    if (finish())
+	fprintf(stderr, finishfailmess);
+    
     /* output statistics */
     record--;
     fprintf(stderr, "\n*** sort statistics ***\n");
@@ -470,14 +609,30 @@ rdtape_eg(
 	    badtrunc, 100.0*badtrunc/record);
     fprintf(stderr, "Records containing 16 bit skew         : %d (%8.4f%%)\n",
 	    bad16, 100.0*bad16/record);
-    bad = badfirst + badlength + badword + badeob + badtrunc + bad16 +
-	(badshort+badlong-recovershort-recoverlong);
-    total = badrecords - (badshort+badlong-recovershort-recoverlong);
+    fprintf(stderr, "Records with missing first length      : %d (%8.4f%%); "
+	    "recoveries: %d\n", badfirstlength, 100.0*badfirstlength/record,
+	    recoverfirstlength);
+    fprintf(stderr, "Records with repeated event length     : %d (%8.4f%%)\n",
+	    badrepeatlength, 100.0*badrepeatlength/record);
+    fprintf(stderr, "Records overrun from previous record   : %d (%8.4f%%); "
+	    "recoveries: %d\n", badoverrun, 100.0*badoverrun/record,
+	    recoveroverrun);
+    fprintf(stderr, "Records overrun from unknown source    : %d (%8.4f%%)\n",
+	    badoverrun2, 100.0*badoverrun2/record);
+    fprintf(stderr, "Records with missing start             : %d (%8.4f%%)\n",
+	    badstart, 100.0*badstart/record);
+    bad = badfirst + badlength + badword + badeob + bad16 +
+	badfirstlength - recoverfirstlength + badoverrun - recoveroverrun +
+	badoverrun2 + (badshort+badlong-recovershort-recoverlong);
+    total = badrecords - (badshort+badlong-recovershort-recoverlong) +
+	recoverfirstlength + badrepeatlength + badstart + recoveroverrun;
     fprintf(stderr, "Total records in stream: %d; good: %d (%8.4f%%);\n"
-	"      recoverable: %d (%8.4f%%); bad: %d (%8.4f%%)\n", record,
-	    (record-bad-total), 100.0*(record-bad-total)/record,
+	    "      recoverable: %d (%8.4f%%); bad: %d (%8.4f%%)\n", record,
+	    (record-badcount), 100.0*(record-badcount)/record,
 	    total, 100.0*total/record, bad, 100.0*bad/record);
     bad = badshort+badlong;
+    if (style == STYLE_MEGHA)
+	bad += dummies[MEGHA_MAX_DUMMY+1];
     total = in_events+bad;
     fprintf(stderr, "Events with word count too short       : %d (%8.4f%%); "
 	    "recoveries: %d\n", badshort,
@@ -487,13 +642,24 @@ rdtape_eg(
 	    100.0*badlong/total, recoverlong);
     fprintf(stderr, "Events recovered by recovery procedures: %d (%8.4f%%)\n",
 	    recovered, 100.0*recovered/total);
+    if (style == STYLE_MEGHA)
+    {
+	for(i=0; i<=MEGHA_MAX_DUMMY; i++)
+	    fprintf(stderr, "Events with %d extra dummy cycle%c       : "
+		    "%d (%8.4f%%)\n", i, (i == 1) ? ' ' : 's', dummies[i],
+		    100.0*dummies[i]/total);
+	fprintf(stderr, "Events with >%d extra dummy cycle%c      : "
+		"%d (%8.4f%%)\n", MEGHA_MAX_DUMMY,
+		(MEGHA_MAX_DUMMY == 1) ? ' ' : 's', dummies[MEGHA_MAX_DUMMY+1],
+		100.0*dummies[MEGHA_MAX_DUMMY+1]/total);
+    }
     fprintf(stderr, "Total events in stream: %d; good: %ld (%8.4f%%);\n"
-	"    bad: %d (%8.4f%%)\n", total, in_events, 100.0*in_events/total,
+	    "    bad: %d (%8.4f%%)\n", total, in_events, 100.0*in_events/total,
 	    bad, 100.0*bad/total);
     fprintf(stderr,"\nfinished sorting data after reading %d records\n",
 	    record);
-      
-    return(filter);
+    
+    return filter;
 }
 
 /*
@@ -733,6 +899,7 @@ write_eg(
  *  functions used in the filtering of data
  *  write any data to end of filtered data block
  */
+/*ARGSUSED*/
 void eg_filt_tail(
 #if NeedFunctionPrototype
 	  short *base, short *fptr)
