@@ -10,36 +10,20 @@
 #include <stdio.h>
 #include <string.h>
 #include <malloc.h>
-#include <errno.h>
-#include <memory.h>
 
 #include <sys/types.h>
 #include <sys/mman.h>
 
-#ifdef SVR4
-#define NeedFunctionPrototype 1
-#else
-#define NeedFunctionPrototype 0
-#endif
-
+#include "sort_def.h"
 #include "sort_thread.h"
 #include "eg.h"
+#include "data_io.h"
 #include "spectra.h"
+#include "spectraint.h"
 
-extern int DBX_val;
-
-/* function defined in eg.c */
-#if NeedFunctionPrototype
-extern int *map_to_spectrum(int, char *, int, int);
-extern int make_1d(int, int, char *, spec_entry_t *);
-extern int make_2d(int, int, char *, spec_entry_t *);
-extern int make_var(int, float, char *, spec_entry_t *);
-extern spec_entry_t *get_spec_entry(int, int);
-#else
-extern int *map_to_spectrum();
-extern int make_1d(), make_2d(), make_var();
-extern spec_entry_t *get_spec_entry();
-#endif
+int make_1d P((int, int, char *, spec_entry_t *));
+int make_2d P((int, int, char *, spec_entry_t *));
+int make_var P((int, float, char *, spec_entry_t *));
 
 int
 read_spectra_data
@@ -225,6 +209,7 @@ spec_entry_t *spec_entry;
 }
 
 /* Almost Identical to make_1d */
+/*ARGSUSED*/
 int
 make_var
 #if NeedFunctionPrototype
@@ -511,6 +496,7 @@ register int channel;
 	return;
     }
 
+    /*LINTED*/
     if (!(ptr = tab_1d[spec_nos])) {
 	fprintf(stderr, "inc1d to undefined 1d spectrum nos %d\n", spec_nos);
 	return;
@@ -550,6 +536,7 @@ register int y_ch;
 	return;
     }
 
+    /*LINTED*/
     if (!(ptr = tab_2d[spec_nos])) {
 	fprintf(stderr, "inc2d to undefined 2d spectrum nos %d\n", spec_nos);
 	return;
@@ -591,6 +578,7 @@ register int value;
 	return;
     }
 
+    /*LINTED*/
     if (!(ptr = tab_1d[spec_nos])) {
 	fprintf(stderr, "incv1d to undefined 1d spectrum nos %d\n", spec_nos);
 	return;
@@ -633,6 +621,7 @@ register int value;
 	return;
     }
 
+    /*LINTED*/
     if (!(ptr = tab_2d[spec_nos])) {
 	fprintf(stderr, "incv2d to undefined 2d spectrum nos %d\n", spec_nos);
 	return;
@@ -676,6 +665,7 @@ register int y_ch;
 	return SORT_FALSE;
     }
 
+    /*LINTED*/
     if (!(ptr = tab_2d[spec_nos])) {
 	fprintf(stderr, "win2d to undefined 2d spectrum nos %d\n", spec_nos);
 	return SORT_FALSE;
@@ -720,6 +710,7 @@ register int value;
 	return;
     }
 
+    /*LINTED*/
     if (!(ptr = tab_1d[spec_nos])) {
 	fprintf(stderr, "set1d to undefined 1d spectrum nos %d\n", spec_nos);
 	return;
@@ -757,6 +748,7 @@ register int value;
 	return;
     }
 
+    /*LINTED*/
     if (!(ptr = tab_2d[spec_nos])) {
 	fprintf(stderr, "set2d to undefined 2d spectrum nos %d\n", spec_nos);
 	return;
@@ -792,6 +784,7 @@ register int channel;
 	return -1;
     }
 
+    /*LINTED*/
     if (!(ptr = tab_1d[spec_nos])) {
 	fprintf(stderr, "val1d to undefined 1d spectrum nos %d\n", spec_nos);
 	return -1;
@@ -826,6 +819,7 @@ register int y_ch;
 	return -1;
     }
 
+    /*LINTED*/
     if (!(ptr = tab_2d[spec_nos])) {
 	fprintf(stderr, "val2d to undefined 2d spectrum nos %d\n", spec_nos);
 	return -1;
@@ -1214,7 +1208,7 @@ double channel;
 double value;
 #endif
 {
-      incv1d(spec_nos, (int) (channel + 0.5), value);
+      incv1d(spec_nos, (int) (channel + 0.5), (int) (value + 0.5));
 }
 
 void
@@ -1232,7 +1226,8 @@ double y_ch;
 double value;
 #endif
 {
-      incv2d(spec_nos, (int) (x_ch + 0.5), (int) (y_ch + 0.5), value);
+      incv2d(spec_nos, (int) (x_ch + 0.5), (int) (y_ch + 0.5),
+	     (int) (value + 0.5));
 }
 
 int
@@ -1264,7 +1259,7 @@ double channel;
 double value;
 #endif
 {
-      set1d(spec_nos, (int) (channel + 0.5), value);
+      set1d(spec_nos, (int) (channel + 0.5), (int) (value + 0.5));
 }
 
 void
@@ -1282,7 +1277,8 @@ double y_ch;
 double value;
 #endif
 {
-      set2d(spec_nos, (int) (x_ch + 0.5), (int) (y_ch + 0.5), value);
+      set2d(spec_nos, (int) (x_ch + 0.5), (int) (y_ch + 0.5),
+	    (int) (value + 0.5));
 }
 
 int
@@ -1360,7 +1356,7 @@ float *channel;
 float *value;
 #endif
 {
-      incv1d(*spec_nos, (int) (*channel + 0.5), *value);
+      incv1d(*spec_nos, (int) (*channel + 0.5), (int) (*value + 0.5));
 }
 
 void
@@ -1378,7 +1374,8 @@ float *y_ch;
 float *value;
 #endif
 {
-      incv2d(*spec_nos, (int) (*x_ch + 0.5), (int) (*y_ch + 0.5), *value);
+      incv2d(*spec_nos, (int) (*x_ch + 0.5), (int) (*y_ch + 0.5),
+	     (int) (*value + 0.5));
 }
 
 void
@@ -1412,7 +1409,7 @@ float *channel;
 float *value;
 #endif
 {
-      set1d(*spec_nos, (int) (*channel + 0.5), *value);
+      set1d(*spec_nos, (int) (*channel + 0.5), (int) (*value + 0.5));
 }
 
 void
@@ -1430,7 +1427,8 @@ float *y_ch;
 float *value;
 #endif
 {
-      set2d(*spec_nos, (int) (*x_ch + 0.5), (int) (*y_ch + 0.5), *value);
+      set2d(*spec_nos, (int) (*x_ch + 0.5), (int) (*y_ch + 0.5),
+	    (int) (*value + 0.5));
 }
 
 int
@@ -1508,7 +1506,7 @@ double *channel;
 double *value;
 #endif
 {
-      incv1d(*spec_nos, (int) (*channel + 0.5), *value);
+      incv1d(*spec_nos, (int) (*channel + 0.5), (int) (*value + 0.5));
 }
 
 void
@@ -1526,7 +1524,8 @@ double *y_ch;
 double *value;
 #endif
 {
-      incv2d(*spec_nos, (int) (*x_ch + 0.5), (int) (*y_ch + 0.5), *value);
+      incv2d(*spec_nos, (int) (*x_ch + 0.5), (int) (*y_ch + 0.5),
+	     (int) (*value + 0.5));
 }
 
 void
@@ -1560,7 +1559,7 @@ double *channel;
 double *value;
 #endif
 {
-      set1d(*spec_nos, (int) (*channel + 0.5), *value);
+      set1d(*spec_nos, (int) (*channel + 0.5), (int) (*value + 0.5));
 }
 
 void
@@ -1578,7 +1577,8 @@ double *y_ch;
 double *value;
 #endif
 {
-      set2d(*spec_nos, (int) (*x_ch + 0.5), (int) (*y_ch + 0.5), *value);
+      set2d(*spec_nos, (int) (*x_ch + 0.5), (int) (*y_ch + 0.5),
+	    (int) (*value + 0.5));
 }
 
 int
