@@ -15,49 +15,25 @@
 #include <fcntl.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <sys/stat.h>
-#include <sys/socket.h>	/* remote tape handling */
-#include <netdb.h>	/*    ditto      */
 #include <stdlib.h>
-#include <malloc.h>
 #include <string.h>
 #include <ctype.h>
 #include <sys/time.h>	/* time functions		*/
 #include <time.h>
-#include <memory.h>	/* memset, memcpy etc   */
-#include <sys/ioctl.h>	/* mt controls          */
 #include <sys/mtio.h>	/* mt controls          */
 
-#ifdef SVR4
-#include <sys/uio.h>
-#define NeedFunctionPrototype 1
-#else
-#define NeedFunctionPrototype 0
-#endif
-
+#include "sort_def.h"
 #include "datatape.h"
+#include "sort_threadint.h"
+#include "data_tape.h"
 
-#ifdef NeedFunctionPrototype
-static void fix2gblimit(int fd);
-typedef int (*IOfunc)(int, char *);
-static int dtread_vr(int, char *);
-static int dtread_fr(int, char *);
-static int dtwrite_vr(int, char *);
-static int dtwrite_fr(int, char *);
-extern int Stop_interupt(int, ...);
-extern int newvol(int fd, char *volume);
-extern int toupperstring( char *s);
-#else
-typedef int (*IOfunc)();
-static void fix2gblimit();
-static int dtread_vr();
-static int dtread_fr();
-static int dtwrite_vr();
-static int dtwrite_fr();
-extern int Stop_interupt();
-extern int newvol();
-extern int toupperstring();
-#endif
+static void fix2gblimit P((int fd));
+static int dtread_vr P((int, char *));
+static int dtread_fr P((int, char *));
+static int dtwrite_vr P((int, char *));
+static int dtwrite_fr P((int, char *));
+static int newvol P((int fd, char *volume));
+static void toupperstring P((char *s));
 
 /* information set in dtopen and dtclose commands */
 struct file_data {
@@ -225,6 +201,9 @@ open_tape_device(
 	return(0);	/* no error */
 }
 /*************************************************************************************/
+
+#if 0 /* this functions is never used */
+
 /* get current file number */
 static int
 tape_getfile(
@@ -239,6 +218,9 @@ tape_getfile(
 	ioctl(fd, MTIOCGET, &mt);
 	return( (int) mt.mt_fileno );
 }
+
+#endif
+
 /*************************************************************************************/
 /*
  * move tape by requested number of data files
@@ -378,7 +360,7 @@ read_ansi_headers(
 	else
 		read_dev.fdata.hdr34 = FALSE;
 	if( verbose ) {
-		fprintf(stderr, "file %0.*s\n", (int) strcspn( read_dev.fdata.filename, " " ),
+		fprintf(stderr, "file %.*s\n", (int) strcspn( read_dev.fdata.filename, " " ),
 			read_dev.fdata.filename);
 		fprintf(stderr, "block length %i, maximum record length %i\n", read_dev.fdata.blen,
 			read_dev.fdata.rlen);
@@ -421,7 +403,7 @@ write_ansi_headers(
 		memcpy(hdr1.blockcount, "000000", 		6);
 	}
 	else {
-		sprintf( temp, "%06.6li", write_dev.fdata.blocks );
+		sprintf( temp, "%6.6li", write_dev.fdata.blocks );
 		memcpy( hdr1.blockcount, temp, 		6);
 	}
 	memcpy(hdr1.tapesys, "DATATAPE-SUN ",		13);
@@ -435,9 +417,9 @@ write_ansi_headers(
 	memcpy(hdr2.block_offset, "00",			2);
 	hdr2.recfmt = write_dev.fdata.format;
 	hdr2.blocked_records = 'B';
-	sprintf( temp, "%05.5i", write_dev.fdata.blen );
+	sprintf( temp, "%5.5i", write_dev.fdata.blen );
 	memcpy( hdr2.blocklen, temp, 5 );
-	sprintf( temp, "%05.5i", write_dev.fdata.rlen );
+	sprintf( temp, "%5.5i", write_dev.fdata.rlen );
 	memcpy( hdr2.reclen, temp, 5 );
 	
 	/* write the headers, followed by 1 file mark */
@@ -543,7 +525,7 @@ ansi_mount(
       int	mode;		/* READ_MODE (0) or WRITE_MODE (1)  or NEWVOL_MODE (2)*/
 #endif
 {
-      char	*volume_temp = "      ";
+      char	volume_temp[6];
       int	iopen;
       int       buffsize;
       int       fd;
@@ -615,7 +597,7 @@ ansi_mount(
       memcpy(volume_temp, volume, volume_l );
       if ( strncasecmp( volume_temp, vol1.label, 6 ) ){
 	    (void) close_tape_device( mode, TRUE);
-	    fprintf(stderr, "dtmount: wrong volume %0.6s\n", vol1.label);
+	    fprintf(stderr, "dtmount: wrong volume %.6s\n", vol1.label);
 	    return (MOUNT_WRONG_TAPE); /* wrong volume label */
       }
       
@@ -882,7 +864,7 @@ ansi_open(
 	    
 	    /* read headers, return error if there is one */
 	    while ( !read_dev.file_open ){
-		  if (Stop_interupt(NULL)) {
+		  if (Stop_interupt(0)) {
 			fprintf(stderr,"datatape: dtopen interupted during open operation\n");
 			return(FOPEN_INTERUPT);
 		  }
@@ -940,6 +922,7 @@ ansi_open(
       return( SUCCESS );
 }
 
+/*ARGSUSED*/
 int
 raw_open(
 #if NeedFunctionPrototype
@@ -1511,18 +1494,31 @@ dtwrite_fr(
 #if NeedFunctionPrototype
 int dtverb(int flag) {verbose = flag; return(0);}
 
+#if 0 /* This functions is never used */
 int dtvers(void) {return( DTVERSION );}
+#endif
       
-int toupperstring( char *s) {	/* converts null-terminated string to upper case */
-      while( *s != '\0' ){ *s = toupper( *s ); s++; } return(0);}
+/* converts null-terminated string to upper case */
+void toupperstring( char *s)
+{
+    for(; *s != '\0'; s++ )
+        *s = toupper( *s );
+}
 
 #else
 int dtverb(flag) int flag; {verbose = flag;}
 
+#if 0 /* This functions is never used */
 int dtvers() {return( DTVERSION );}
+#endif
 
-int toupperstring( s ) char *s; {	/* converts null-terminated string to upper case */
-      while( *s != '\0' ){ *s = toupper( *s ); s++; } }
+/* converts null-terminated string to upper case */
+void toupperstring(s)
+char *s
+{
+    for(; *s != '\0'; s++ )
+        *s = toupper( *s );
+}
 
 #endif
       
@@ -1530,6 +1526,7 @@ int toupperstring( s ) char *s; {	/* converts null-terminated string to upper ca
 /*
  *   rewind a mounted tape
  */
+#if 0 /* This function is never used */
 void
 dtrewin(
 #if NeedFunctionPrototype
@@ -1549,6 +1546,7 @@ dtrewin(
       }
       return;
 }
+#endif
 
 /********************************************************/
 /*	Newvol() - write a volume label to the tape	*/
