@@ -13,44 +13,29 @@
 #include <stdio.h>
 #include <string.h>
 #include <malloc.h>
-#include <errno.h>
-#include <memory.h>
 #include <stdlib.h>
+#include <fcntl.h>
 
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <sys/mman.h>
 
-#ifdef SVR4
-#define NeedFunctionPrototype 1
-#else
-#define NeedFunctionPrototype 0
-#endif
-
-#include "sort_thread.h"
+#include "sort_def.h"
+#include "sort_threadint.h"
 #include "eg.h"
+#include "read_spec.h"
+#include "sort_main.h"
+#include "sdb.h"
 
-extern int        DBX_val;
-
-/* function defined in eg.c */
-#if NeedFunctionPrototype
-extern int             *map_spectrum(int , char *, int , int);
-extern spec_entry_t    *update_shared_list(int, int, float, char *, int);
-static  spec_1d_t      *get_1d_spectrum(char *, int, int);
-static  spec_2d_t      *get_2d_spectrum(char *, int, int);
-extern int             *remap_1d_spectrum(spec_1d_t *p1d, int newsize);
-extern int             *remap_2d_spectrum(spec_2d_t *p1d, int newsize);
-extern int              msync_1d(spec_1d_t *ptr, int flags);
-extern int              msync_2d(spec_2d_t *ptr, int flags);
-#else
-extern int             *map_spectrum();
-extern spec_entry_t    *update_shared_list();
-static  spec_1d_t      *get_1d_spectrum();
-static  spec_2d_t      *get_2d_spectrum();
-extern int             *remap_1d_spectrum();
-extern int             *remap_2d_spectrum();
-extern int              msync_1d();
-extern int              msync_2d();
-#endif
+spec_1d_t* get_1d_spectrum P((char *, int, int));
+spec_2d_t* get_2d_spectrum P((char *, int, int));
+int load_SDB_spectrum P((int *, char *, int, int, int));
+int load_XY_spectrum P((FILE *, char *, int, int, int));
+int load_EG_spectrum P((int *, char *, int, int, int));
+int make_1d P((int, float, char *));	
+int make_2d P((int, float, char *));	
+int msync_1d P((spec_1d_t *, int));
+int msync_2d P((spec_2d_t *, int));
 
 int
 read_spectra_file(
@@ -70,17 +55,9 @@ read_spectra_file(
 	FILE *fd;		/* file descriptor */
 	int ret_val;		/* function return value */
 	char *spectra_file;     /* name of file where spectra info is expected */
-	char *type;             
+/*	char *type;             */
 	
-#if NeedFunctionPrototype
-	int (*func)(int, float, char *);          /* pointer to function that does the work */
-	extern int make_1d(int, float, char *),
-	           make_2d(int, float, char *),
-	           make_var(int, float, char *);
-#else
-	int (*func)();          
-	extern int make_1d(), make_2d(), make_var();
-#endif
+	int (*func) P((int, float, char *));
 
 /* create expected name of spectra file */	
 	spectra_file = malloc(strlen(spectra_proc)+6);
@@ -106,19 +83,19 @@ read_spectra_file(
 			switch(*++p) {
 				case 'o': case 'O':
 					func = make_1d;
-					type = "1D spectrum";
+/*					type = "1D spectrum"; */
 					/*fprintf(stderr,"1-D spectra\n");*/
 					break;
 				
 				case 't': case 'T':
 					func = make_2d;
-					type = "2D spectrum";
+/*					type = "2D spectrum"; */
 					/*fprintf(stderr,"2-D spectra\n");*/
 					break;
 				
 				case 'v': case 'V':
 					/*fprintf(stderr,"Variables\n");*/
-					type = "Variable";
+/*					type = "Variable"; */
 					func = make_var;
 					break;
 					
@@ -373,42 +350,32 @@ get_nametab_var()
 
 /*---------------------------------------------------*/
 void
-clear_1d(
+clear_1d
 #if NeedFunctionPrototype
-	 spec_1d_t	*ptr)
+(spec_1d_t *ptr)
 #else
-      ptr)
-      spec_1d_t	        *ptr;
+(ptr)
+spec_1d_t *ptr;
 #endif
 {
-	register int	i;
-	register int	size = ptr->sp->size;
-	register int	*data = ptr->data;
-
-	for (i=0; i<size; i++)
-		*data++ = 0;
-	if ( DBX_val >= 5) fprintf(stderr,"cleared 1d spectrum %d %s\n",
-				ptr->nos,ptr->sp->name);
-	return;
+    memset(ptr->data, 0, ptr->sp->size*sizeof(int));
+    if ( DBX_val >= 5) fprintf(stderr,"cleared 1d spectrum %d %s\n",
+			       ptr->nos,ptr->sp->name);
+    return;
 }
 void
-clear_2d(
+clear_2d
 #if NeedFunctionPrototype
-	 spec_2d_t	*ptr)
+(spec_2d_t *ptr)
 #else
-      ptr)
-      spec_2d_t	        *ptr;
+(ptr)
+spec_2d_t *ptr;
 #endif
 {
-	register int	i;
-	register int 	size = ptr->sp->size*ptr->sp->size;
-	register int	*data = ptr->data;
-
-	for (i=0; i<size; i++)
-		*data++ = 0;
-	if ( DBX_val >= 5) fprintf(stderr,"cleared 2d spectrum %d %s\n",
-				ptr->nos,ptr->sp->name);
-	return;
+    memset(ptr->data, 0, ptr->sp->size*ptr->sp->size*sizeof(int));
+    if ( DBX_val >= 5) fprintf(stderr,"cleared 2d spectrum %d %s\n",
+			       ptr->nos,ptr->sp->name);
+    return;
 }
 
 int
@@ -430,7 +397,7 @@ Clear_1d_spectra(
 	    /* clear all the one dimensional spectra */
 	    while ( p1d ) {
 		  clear_1d(p1d);
-		  (void) msync_1d(p1d, 0);
+		  (void) msync_1d(p1d, MS_ASYNC);
 		  p1d = p1d->next;
 	    }
 	    fprintf(stderr,"server: all 1d spectra cleared...\n");
@@ -483,7 +450,7 @@ Clear_2d_spectra(
 	    while ( p2d ) {
 		  if ( p2d->win == 0) { /* don't clear window spectra */
 			clear_2d(p2d);
-			(void) msync_2d(p2d, 0);
+			(void) msync_2d(p2d, MS_ASYNC);
 		  }
 		  p2d = p2d->next;
 	    }
@@ -796,103 +763,207 @@ delete_spectra(
 	tab_1d).
 */
 int 
-load_spectrum_file(
+load_spectrum_file
 #if NeedFunctionPrototype
-		   char 	*dir_name,
-		   char	        *file_name,
-		   int          IO_state)
+(char 	*dir_name,
+char	        *file_name,
+int          IO_state)
 #else
-      dir_name,file_name,IO_state)
-      char 	*dir_name;
-      char	*file_name;
-      int       IO_state;
+(dir_name,file_name,IO_state)
+char 	*dir_name;
+char	*file_name;
+int       IO_state;
 #endif
 {
-	char		sbuf[MED_SUNSORT_BUFFER+32];
-	char		*cp;
-	char		name[NAME_SIZE];		/* read in from file name */
-	int		nos;				/* read in from file name */
-	int		type;				/* read in from file name */
-	int		ret_code;
-	int		id;				/* for use in SDB format data */
-	FILE            *fp;                            /* for use in X-Y format data */
+    char sbuf[MED_SUNSORT_BUFFER+32];
+    char *cp;
+    char name[NAME_SIZE];		/* read in from file name */
+    int nos;				/* read in from file name */
+    int type;				/* read in from file name */
+    int ret_code;
+    int id;				/* for use in SDB format data */
+    FILE *fp;                           /* for use in X-Y format data */
+    char *myargv[4], *tmpspec = NULL;
 	
-#if NeedFunctionPrototype
-	extern int	load_SDB_spectrum(int *id, char *name, int type, int nos, int add);
-	extern int	load_XY_spectrum(FILE *fp, char *name, int type, int nos, int add);
-#else
-	extern int	load_SDB_spectrum();
-	extern int	load_XY_spectrum();
-#endif
+    /* See what file type has been requested */
+    id = strlen(file_name);
+    if (!strcmp(file_name + id - 3, ".gz"))
+	myargv[0] = "gunzip";
+    else if (!strcmp(file_name + id - 2, ".Z"))
+	myargv[0] = "uncompress";
+    else
+	myargv[0] = NULL;
+    if (myargv[0] != NULL)
+    {
+	int fd;
+	pid_t pid;
 
-	/* decode file name to obtain data on where to store spectrum info */
-	strcpy(sbuf,file_name);
-        if ((cp = strrchr(sbuf, '_')) != NULL)
-            *cp = ' ';
-        if ((cp = strrchr(sbuf, '_')) != NULL)
-	    *cp = ' ';
-/*
-	for (cp=sbuf; *cp != '\0'; cp++)
-		if ( *cp == '_') *cp = ' ';
-*/
-	if ( sscanf(sbuf,"%s %dd %d",name,&type,&nos) != 3) {
-		fprintf(stderr,"server: error decoding filename %s during load\n",file_name);
-		return(SORT_FAIL);
-	}
-
-	if ( DBX_val >= 5)
-            fprintf(stderr, "server: load spectrum with IO flags = 0x%x\n" 
-                    "        name=%s, type=%dd, nos=%d\n", IO_state, name,
-                    type, nos);
-	
-/* generate fully specified path name for spectrum file */
 	sprintf(sbuf,"%s/%s",dir_name,file_name);
+    
+	myargv[1] = "-c";
+	myargv[2] = sbuf;
+	myargv[3] = NULL;
 
-/* read file and place data into spectrum storage */
-	switch(IO_state & FMT_MASK) {
-	   case SDB_TYPE:
-/* open connection to sdb file */
-	      if (sname(&id,sbuf,strlen(sbuf)) ) {
-		    fprintf(stderr,"server: failed to open SDB format file %s for reading\n",sbuf);
-		    return(SORT_FAIL);
-	      }
-	      ret_code = load_SDB_spectrum(&id,name,type,nos,IO_state & LOAD_N_ADD);
-	      (void) sname(&id,sbuf,0);
-	      break;
-
-	   case XY_TYPE:
-	      if ( (fp = fopen(sbuf,"r")) == NULL) {
-		    fprintf(stderr,"server: failed to open X-Y format file %s for reading\n",sbuf);
-		    return(SORT_FAIL);
-	      }
-	      ret_code = load_XY_spectrum(fp,name,type,nos,IO_state & LOAD_N_ADD);
-	      (void) fclose(fp);
-	      break;
-
-	   case EG_TYPE:
-	      eexpt(&id);
-	      if ( id != 0) {
-	            fprintf(stderr,"server: failed to connect to EG experiment\n");
-		    return(SORT_FAIL);
-	      }
-	      if (ename(&id,sbuf,strlen(sbuf)) ) {
-		    fprintf(stderr,"server: failed to open EG  format file %s for reading\n",sbuf);
-		    return(SORT_FAIL);
-	      }
-	      ret_code = load_EG_spectrum(&id,name,type,nos,IO_state & LOAD_N_ADD);
-	      (void) ename(&id,sbuf,0);
-	      break;
-
-	   default:
-	      fprintf(stderr,"server: unknown file type 0x%x during load file operation",IO_state & FMT_MASK);
-	      return(SORT_FAIL);
+	tmpspec = tempnam(NULL, NULL);
 
+	if ((fd = open(tmpspec, O_WRONLY | O_CREAT, 0666)) == -1)
+	{
+	    perror("server");
+	    fprintf(stderr, "server: Unable to create temporary file for "
+		    "decompression.\n");
+	    free(tmpspec);
+	    return SORT_FAIL;
 	}
-/* write out full file name */
-	fprintf(stderr," %s\n",sbuf);
-	return(ret_code);
+
+	switch(pid = fork1())
+	{
+	case -1:
+	    fprintf(stderr, "server: Failed to fork decompression process.\n");
+	    close(fd);
+	    unlink(tmpspec);
+	    free(tmpspec);
+	    return SORT_FAIL;
+	case 0:
+	    dup2(fd, 1);
+	    /* clean up streams if they are open */
+	    for (fd=3; fd<getdtablesize(); fd++)
+		close(fd);
+	    
+	    /* start new process */
+	    execvp(myargv[0], myargv);
+	    fprintf(stderr,"server: failed to exec decompression process\n");
+	    _exit(-1);
+	}
+
+	close(fd);
+	while(waitpid(pid, &fd, 0) != pid)
+	    ;
+	if (!WIFEXITED(fd) || WEXITSTATUS(fd) != 0)
+	{
+	    fprintf(stderr, "server: Decompression process failed.\n");
+	    unlink(tmpspec);
+	    free(tmpspec);
+	    return SORT_FAIL;
+	}
+    }
+    
+    /* decode file name to obtain data on where to store spectrum info */
+    strcpy(sbuf,file_name);
+
+    if ((cp = strrchr(sbuf, '_')) != NULL)
+	*cp = ' ';
+    if ((cp = strrchr(sbuf, '_')) != NULL)
+	*cp = ' ';
+/*
+  for (cp=sbuf; *cp != '\0'; cp++)
+  if ( *cp == '_') *cp = ' ';
+*/
+    
+    if ( sscanf(sbuf,"%s %dd %d",name,&type,&nos) != 3) {
+	fprintf(stderr,"server: error decoding filename %s during load\n",
+		file_name);
+	if (tmpspec != NULL)
+	{
+	    unlink(tmpspec);
+	    free(tmpspec);
+	}
+	return SORT_FAIL;
+    }
+    
+    if ( DBX_val >= 5)
+	fprintf(stderr, "server: load spectrum with IO flags = 0x%x\n"
+		"        name=%s, type=%dd, nos=%d\n", IO_state, name,
+		type, nos);
+    
+    /* generate fully specified path name for spectrum file */
+    if (tmpspec == NULL)
+    {
+	sprintf(sbuf,"%s/%s",dir_name,file_name);
+	tmpspec = sbuf;
+    }
+    
+    /* read file and place data into spectrum storage */
+    switch(IO_state & FMT_MASK) {
+    case SDB_TYPE:
+	/* open connection to sdb file */
+	if (sname(&id, tmpspec, strlen(tmpspec)) ) {
+	    fprintf(stderr,"server: failed to open SDB format file %s for "
+		    "reading\n", tmpspec);
+	    if (tmpspec != sbuf)
+	    {
+		unlink(tmpspec);
+		free(tmpspec);
+	    }
+	    return SORT_FAIL;
+	}
+	ret_code = load_SDB_spectrum(&id,name,type,nos,IO_state & LOAD_N_ADD);
+	(void) sname(&id,tmpspec,0);
+	break;
+	
+    case XY_TYPE:
+	if ( (fp = fopen(tmpspec,"r")) == NULL) {
+	    fprintf(stderr,"server: failed to open X-Y format file %s for "
+		    "reading\n",tmpspec);
+	    if (tmpspec != sbuf)
+	    {
+		unlink(tmpspec);
+		free(tmpspec);
+	    }
+	    return SORT_FAIL;
+	}
+	ret_code = load_XY_spectrum(fp,name,type,nos,IO_state & LOAD_N_ADD);
+	(void) fclose(fp);
+	break;
+	
+    case EG_TYPE:
+	eexpt(&id);
+	if ( id != 0) {
+	    fprintf(stderr,"server: failed to connect to EG experiment\n");
+	    if (tmpspec != sbuf)
+	    {
+		unlink(tmpspec);
+		free(tmpspec);
+	    }
+	    return SORT_FAIL;
+	}
+	      if (ename(&id,tmpspec,strlen(tmpspec)) ) {
+		  fprintf(stderr,"server: failed to open EG format file %s "
+			  "for reading\n",tmpspec);
+		  if (tmpspec != sbuf)
+		  {
+		      unlink(tmpspec);
+		      free(tmpspec);
+		  }
+		  return SORT_FAIL;
+	      }
+	      ret_code = load_EG_spectrum(&id,name, type, nos,
+					  IO_state & LOAD_N_ADD);
+	      (void) ename(&id,tmpspec,0);
+	      break;
+	      
+    default:
+	fprintf(stderr,"server: unknown file type 0x%x during load file "
+		"operation", IO_state & FMT_MASK);
+	if (tmpspec != sbuf)
+	{
+	    unlink(tmpspec);
+	    free(tmpspec);
+	}
+	return SORT_FAIL;
+	
+    }
+    if (tmpspec != sbuf)
+    {
+	unlink(tmpspec);
+	free(tmpspec);
+    }
+    /* write out full file name */
+    fprintf(stderr," %s/%s\n", dir_name, file_name);
+    return(ret_code);
 }
+
 #define MAX(a,b) ((a) > (b)) ? (a) : (b)
+
 int 
 load_SDB_spectrum(
 #if NeedFunctionPrototype
