@@ -10,29 +10,33 @@
 #include <string.h>
 #include <dirent.h>
 #include <signal.h>
-#include <memory.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <ctype.h>
 
 #include <sys/param.h>
 #include <sys/types.h>
-#include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/time.h>
-#include <sys/resource.h>
 
 #include <thread.h>
 #include <synch.h>
-
-#include <errno.h>
 #include <fcntl.h>
 
-#include "sort_extern.h"
+#include "sort_def.h"
+#include "sort_threadint.h"
 #include "sort_mem.h"
 #include "datatape.h"
 #include "../sunsort_lib/formats.h"
 #include "demon.h"
+#include "read_spec.h"
+#include "discsubs.h"
+#include "sort_main.h"
+#include "sort_notify.h"
+#include "sort_comms.h"
+#include "data_tape.h"
+#include "data_disk.h"
+#include "data_remote.h"
 
 /**************************************************************************************************/
 
@@ -52,19 +56,13 @@ typedef struct IO_format {
 } IO_format_t;
 
 /* known format types */
-static IO_format_t   formats[] = {FMT_PARAM_LIST, NULL};
+static IO_format_t   formats[] = {FMT_PARAM_LIST,
+				  "", 0, 0, 0, 0, 0, 0, 0, 0, 0};
 
 /* current format state */
 static IO_format_t   IOfmt;
 
-/**************************************************************************************************/
-
-/* infomation concerning current sort state */
-extern sort_IO_t 	*sort_io_data;
-extern dirs_IO_t	*dirs_io_data;
-extern unsigned long    *sorted_records;
-extern unsigned long    *filtered_records;
-extern int FDIN, FDOUT, FDDAT, FDBAT;
+static char actionchar = 'Y';
 
 /**************************************************************************************************/
 /*
@@ -88,7 +86,7 @@ open_sort_file(
 	    fprintf(stderr,"server: open_sort_file called with arg %s\n",opt_str);
 
 /* check that not trying to sort */      
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -188,7 +186,7 @@ close_sort_file(
 	    fprintf(stderr,"server: close_sort_file called with arg %s\n",opt_str);
       
 /* check that not trying to sort */      
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -253,7 +251,7 @@ open_filt_file(
       if (DBX_val >= 5)
 	    fprintf(stderr,"server: open_filt_file called with arg %s\n",opt_str);
 
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -320,7 +318,7 @@ close_filt_file(
       if (DBX_val >= 5)
 	    fprintf(stderr,"server: close_filt_file called with arg %s\n",opt_str);
 
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -371,19 +369,12 @@ sort_data(
 #endif
 {
       int skip,stop;
-      extern int skip_records, sort_records;
 
-#if NeedFunctionPrototype
-      extern void *sort_thread( void *);
-#else
-      extern void *sort_thread();
-#endif
-      
       if (DBX_val >= 5)
 	    fprintf(stderr,"server: sort_data called with arg %s\n",opt_str);
 
 /* check that not already trying to sort */      
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -425,7 +416,7 @@ sort_data(
 	    return(SORT_FALSE);
       }
 
-      if (thr_create(NULL, NULL, sort_thread, (void *) sort_io_data, THR_DETACHED, NULL )) {
+      if (thr_create(NULL, 0, sort_thread, (void *) sort_io_data, THR_DETACHED, NULL )) {
 	    (void) Sorting_data(SORT_SET, SORT_FALSE);
 	    sprintf(reply,"error while trying to start main sort thread\n");
 	    return(SORT_FALSE);
@@ -459,7 +450,7 @@ tapemount(
       if (DBX_val >= 1)
 	    dtverb(1);
       
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -531,7 +522,7 @@ newWtape(
       if (DBX_val >= 1)
 	    dtverb(1);
       
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -593,7 +584,7 @@ tapeumount(
       if (DBX_val >= 5)
 	    fprintf(stderr,"server: tapeumount called with arg %s\n",opt_str);
       
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -662,7 +653,7 @@ eject_func(
       if (DBX_val >= 5)
 	    fprintf(stderr,"server: eject called with arg %s\n",opt_str);
       
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -693,7 +684,7 @@ tapemove(
       if (DBX_val >= 5)
 	    fprintf(stderr,"server: tapemove called with arg %s\n",opt_str);
       
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -873,8 +864,7 @@ load_spectra(
 /*
  *  send data to 1d display process
  */
-#define MAX_SPEC_DISPLAY 4
-int     displayed_spectrum[MAX_SPEC_DISPLAY+2];
+int displayed_spectrum[MAX_SPEC_DISPLAY+2];
 
 int
 Display_1d(
@@ -962,7 +952,8 @@ display_1d(
 {
       if (DBX_val >= 5)
 	    fprintf(stderr,"server: display_1d called with arg %s\n",opt_str);
-      (void) strcpy(pshm->com_str,"DISPLAY 1");
+      (void) strcpy(pshm->com_str,"DISPLAY 1"); /*set xvgr exec process from menu*/
+      pshm->com_str[6] = actionchar;
       return(Display_1d(opt_str, reply));
 }
 
@@ -1155,7 +1146,7 @@ setwindow(
       if (DBX_val >= 5)
 	    fprintf(stderr,"server: setwindow called with arg %s\n",opt_str);
       
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -1384,7 +1375,7 @@ change_var(
       if (DBX_val >= 5)
 	    fprintf(stderr,"server: change_var called with arg %s\n",opt_str);
       
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -1413,6 +1404,7 @@ change_var(
       return(SORT_FALSE);
 }
 
+/*ARGSUSED*/
 int
 update_gui(
 #if NeedFunctionPrototype
@@ -1432,6 +1424,7 @@ update_gui(
 /*
  * display on stderr the current value of the variables var(n) used in sorting
  */
+/*ARGSUSED*/
 int
 print_vars(
 #if NeedFunctionPrototype
@@ -1546,7 +1539,7 @@ batch_read(
       if (DBX_val >= 5)
 	    fprintf(stderr,"server: batch_read called with arg %s\n",opt_str);
 
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -1593,7 +1586,7 @@ end_batch(
 	    FDDAT = FDOUT;
 	    FDBAT = STDIN_FILENO;
 	    /* wait for sort thread to exit or timeout if sorting */
-	    while (Sorting_data(NULL) == SORT_TRUE) {
+	    while (Sorting_data(0) == SORT_TRUE) {
 		  sleep(1);
 	    }
 	    /* if gui is alive signal that batch mode has stopped */
@@ -1607,6 +1600,7 @@ end_batch(
 /*
  *  exit sorting program and terminate any associated processes
  */
+/*ARGSUSED*/
 int
 exit_sunsort(
 #if NeedFunctionPrototype
@@ -1663,6 +1657,9 @@ exit_sunsort(
 
       (void) signal(SIGTERM, SIG_DFL);
       exit(0);
+#ifdef lint
+      return SORT_TRUE;
+#endif
 }
 
 /*
@@ -1757,43 +1754,44 @@ refresh_func(
  *  set up IOfmt structure with info corresponding to predefined EbyE data format
  */
 int
-set_format(
+set_format
 #if NeedFunctionPrototype
-	   const char *opt_str)
+(const char *opt_str)
 #else
-      opt_str)
-      char     *opt_str;
+(opt_str)
+char     *opt_str;
 #endif
 {
-      char format_name[BUFSIZ];
-      int i;
-      
-      if (DBX_val >= 5)
-	    fprintf(stderr,"server: set_format function called with args => %s\n",opt_str);
-      
+    char format_name[BUFSIZ];
+    int i;
+    
+    if (DBX_val >= 5)
+	fprintf(stderr,"server: set_format function called with args => %s\n",
+		opt_str);
+    
 /* check that not trying to sort */      
-      if (Sorting_data(NULL) != SORT_FALSE) {
-	    fprintf(stderr,"invalid command while sorting\n");
-	    return(SORT_FALSE);
-      }
-
+    if (Sorting_data(0) != SORT_FALSE) {
+	fprintf(stderr,"invalid command while sorting\n");
+	return(SORT_FALSE);
+    }
+    
 /* decode option string */
-      if ( sscanf(opt_str,"%s",format_name) != 1) {
-	    fprintf(stderr,"invalid set_format argument %s\n",opt_str);
-	    return(SORT_FALSE);
-      }
-
+    if ( sscanf(opt_str,"%s",format_name) != 1) {
+	fprintf(stderr,"invalid set_format argument %s\n",opt_str);
+	return(SORT_FALSE);
+    }
+    
 /* choose one of the formats */
-      for (i=0; strlen(formats[i].format_name) != 0; i++) {
-	    if (strncmp(format_name, formats[i].format_name,FORMAT_NAME_LEN) == 0) {
-		  (void) memcpy(&IOfmt, &formats[i], sizeof(IO_format_t));
-		  strcpy(sort_io_data->decoder, formats[i].format_name);
-		  return(SORT_TRUE);
-	    }
-      }
-
-      fprintf(stderr,"Unknown format type %s\n",format_name);
-      return(SORT_FALSE);
+    for (i=0; strlen(formats[i].format_name) != 0; i++) {
+	if (!strncmp(format_name, formats[i].format_name, FORMAT_NAME_LEN)) {
+	    (void) memcpy(&IOfmt, &formats[i], sizeof(IO_format_t));
+	    strcpy(sort_io_data->decoder, formats[i].format_name);
+	    return(SORT_TRUE);
+	}
+    }
+    
+    fprintf(stderr,"Unknown format type %s\n",format_name);
+    return(SORT_FALSE);
 }
 
 /*
@@ -1870,7 +1868,7 @@ set_sortenv(
 	    fprintf(stderr,"server: set_sortenv function called with args => %s\n",opt_str);
 
 /* check that not trying to sort */      
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -1948,3 +1946,87 @@ print_sortenv(
       sprintf(reply,"\n");
       return(SORT_TRUE);
 }
+
+int
+display_action
+#if NeedFunctionPrototype
+(char *opt_str, char *reply)
+#else
+(opt_str, reply)
+char     *opt_str;
+char     *reply;
+#endif
+{
+    char *p = opt_str;
+
+    while(*p == ' ' || *p == '\t')
+	p++;
+
+    if (DBX_val >= 5)
+	fprintf(stderr,"server: Display_action called with arg %s\n",opt_str);
+
+    if (!strcasecmp(p, "display"))
+    {
+	actionchar = 'Y';
+	sprintf(reply, "Now just displaying spectra.\n");
+	return SORT_TRUE;
+    }
+
+    if (!strcasecmp(p, "mstick") || !strcasecmp(p, "matchstick"))
+    {
+	actionchar = 'M';
+	sprintf(reply, "Now auto-matchsticking displayed spectra.\n");
+	return SORT_TRUE;
+    }
+
+    if (!strcasecmp(p, "pkfind") || !strcasecmp(p, "peakfind"))
+    {
+	actionchar = 'P';
+	sprintf(reply, "Now auto-peakfinding displayed spectra.\n");
+	return SORT_TRUE;
+    }
+
+    if (!strcasecmp(p, "Paper"))
+    {
+	actionchar = 'E';
+	sprintf(reply, "Now auto-writing Phys Rev Lett! Hit display!\n");
+	return SORT_TRUE;
+    }
+
+    sprintf(reply, "Unknown display action \"%s\".\n", p);
+    return SORT_FALSE;
+}
+
+/*
+ *   write down pipe connected to sort process
+ */
+int
+write_sort
+#if NeedFunctionPrototype
+(char *opt_str, char *reply)
+#else
+(opt_str, reply)
+char     *opt_str;
+char     *reply;
+#endif
+{
+    int i;
+
+    if (sort_proc_OK != SORT_TRUE) {
+	sprintf(reply, "server: attempt to communicate with nonexistent "
+		"sort process\n");
+	return SORT_FALSE;
+    }
+    i = strlen(opt_str);
+    opt_str[i] = '\n';
+    if (write(FDSUSEROUT, opt_str+1, i) != i)
+    {
+	sprintf(reply, "server: attempt to communicate with sort process "
+		"failed\n");
+	return SORT_FALSE;
+    }
+
+    *reply = '\0';
+    return SORT_TRUE;
+}
+
