@@ -6,15 +6,10 @@
  *
  */
 
-
-#define SUNSORT_MAIN
-
 #include <stdio.h>
 #include <string.h>
 #include <dirent.h>
 #include <signal.h>
-#include <memory.h>
-#include <fcntl.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <ctype.h>
@@ -28,10 +23,65 @@
 
 #include <errno.h>
 
-#include "sort_thread.h"
-#include "sort_extern.h"
+#include "sort_def.h"
+#include "sort_threadint.h"
 #include "sort_mem.h"
 #include "version.h"
+#include "read_spec.h"
+#include "discsubs.h"
+#include "sort_main.h"
+#include "sort_notify.h"
+#include "sort_comms.h"
+#include "data_tape.h"
+#include "data_disk.h"
+#include "data_remote.h"
+#include "demon.h"
+
+void read_args P((int, char** argv));
+void main_event_loop P((void));
+int decode_command P((char *, char *));
+char *make_tmp_dir P((void));
+int debugsort P((char *, char *));
+int editsort P((char *, char *));
+int help_func P((char *, char *));
+int killsort P((char *, char *));
+int loadsort P((char *, char *));
+int loadspec P((char *, char *));
+int makesort P((char *, char *));
+int status_func P((char *, char *));
+void check_children P((void));
+int start_sortproc P((char *, int, char **));
+
+/* I/O streams */
+int FDIN = STDIN_FILENO;        /* user command input stream (gui) */
+int FDOUT = STDOUT_FILENO;      /* user command output stream (gui) */
+int FDDAT = STDOUT_FILENO;      /* batch data output stream (if !batch_mode 
+				   connected to gui) */
+int FDBAT = STDIN_FILENO;       /* batch command input stream */
+int FDSIN = STDIN_FILENO;       /* Input stream from sort process */
+int FDSOUT = STDOUT_FILENO;     /* Output stream from sort process */
+int FDSUSEROUT = STDOUT_FILENO; /* Output stream to stdin on sort process */
+
+int poll_period = POLL_PERIOD;
+
+/* graphics process information */
+int sort_proc_OK = SORT_FALSE;
+int graf_proc1_OK = SORT_FALSE;
+int graf_proc2_OK = SORT_FALSE;
+int nogui = SORT_FALSE;
+int nograf = SORT_FALSE;
+
+pid_t sort_pid;
+pid_t graf_1d_pid;
+pid_t graf_2d_pid;
+pid_t gui_pid;
+
+/* other status information */
+int refresh_time = REFRESH_OFF;
+int batch_mode = SORT_FALSE;
+char *tmp_spectrum_dir = NULL;
+char *sunsort_proc = NULL;
+int DBX_val = 0;
 
 #ifndef MAX
 #define MAX(x,y) ((int)(x) < (int)(y)) ? (y) : (x)
@@ -40,13 +90,7 @@
 #define MIN(x,y) ((int)(x) > (int)(y)) ? (y) : (x)
 #endif
 
-/* infomation concerning current sort state */
-extern sort_IO_t 	*sort_io_data;
-extern dirs_IO_t	*dirs_io_data;
-extern unsigned long    *sorted_records;
-extern unsigned long    *filtered_records;
-
-/**************************************************************************************/
+/****************************************************************************/
 
 void
 main(
@@ -109,6 +153,7 @@ main(
 	    sort_proc_OK = SORT_FALSE;
 	    FDSIN = STDIN_FILENO;
 	    FDSOUT = STDOUT_FILENO;
+	    FDSUSEROUT = STDOUT_FILENO;
       }
       
 /*  start graphical user interface process */      
@@ -224,6 +269,25 @@ start_graphics(
       return;
 }
 
+/****************** sjah 16/1/95 ********************************/
+/*
+ *  getdtablesize() is a BSD function
+ *  keep name but use Solaris equivalent if SVR4
+ */
+#ifdef SVR4
+
+int getdtablesize(void)
+{
+    struct rlimit rlp;
+    if ( getrlimit(RLIMIT_NOFILE, &rlp) == -1)
+	return(-1);
+    else
+	return((int) rlp.rlim_cur);
+}
+#endif
+
+
+
 /******************************************************************************/
 /*
  *  this function is used by background threads as well
@@ -254,8 +318,8 @@ do_process(
 	        fprintf(stderr,"server: failed to fork %s process\n",process);
 		return(SORT_FAIL);
 	    case 0:
-		/* clean up on first 32 streams if they are open */
-		for (i=3; i<32; i++)
+		/* clean up streams if they are open */
+		for (i=3; i<getdtablesize(); i++)
 		      (void) close(i);
 		execvp(process,argv);
 		fprintf(stderr,"server: failed to exec %s process\n",process);
@@ -271,183 +335,227 @@ do_process(
 /**************************************************************************************************/
 
 int
-do_pipe_process(
+do_pipe_process
 #if NeedFunctionPrototype
-	   char           *process,
-	   char           **my_argv,
-	   int            *streams)
+(char *process, char **my_argv, int ns, int *streams)
 #else
-      process,my_argv,streams)
-      char           *process;
-      char           **my_argv;
-      int            *streams;
+(process, my_argv, ns, streams)
+char *process;
+char **my_argv;
+int ns, *streams;
 #endif
 {
-      int pid;
-      int i, j;
-      int fd[2][2];
+    int pid;
+    int i, j;
+    int (*fd)[2];
 
 /* diagnostics output */      
-      if (DBX_val >= 5) {
-	    fprintf(stderr,"server: do_pipe_process(%s",process);
-	    for (i=0; my_argv[i] != NULL; i++)
-		  fprintf(stderr," ,%s",my_argv[i]);
-	    fprintf(stderr,")\n");
-      }
+    if (DBX_val >= 5)
+    {
+	fprintf(stderr,"server: do_pipe_process(%s",process);
+	for (i=0; my_argv[i] != NULL; i++)
+	    fprintf(stderr," ,%s",my_argv[i]);
+	fprintf(stderr,") with streams: ");
+	for(i = 0; i < ns; i++)
+	    fprintf(stderr, " %d", streams[i]);
+	fprintf(stderr, " connected\n");
+    }
+
+    if ((fd = calloc(ns, sizeof(int [2]))) == NULL)
+    {
+	fprintf(stderr, "Insufficient memory to create process.\n");
+	return 0;
+    }
       
-      /* create pipe for communications between server and gui */      
-      if (pipe(fd[0]) != 0) {
+    /* create pipe for communications between server and gui */
+
+    for(i=0; i<ns; i++)
+    {
+	if (pipe(fd[i]) != 0) {
 	    perror("server");
-	    return(0);
-      }
-      if (pipe(fd[1]) != 0) {
-	    perror("server");
-	    (void) close(fd[0][0]);
-	    (void) close(fd[0][1]);
-	    return(0);
-      }
-      
-/* start graphical user interface */
-      switch(pid = fork1()) {
-	  case -1:
-	    fprintf(stderr,"server: failed to fork %s process\n",process);
-	    for(i=0;i<2;i++) {
-		  for(j=0;j<2;j++){
-			(void) close(fd[i][j]);
-		      }
+	    for(j=0; j<i; j++)
+	    {
+		close(fd[j][0]);
+		close(fd[j][1]);
 	    }
-	    return(0);
-	  case 0:
-	    (void) close(fd[1][0]);
-	    (void) close(fd[0][1]);
-	    /* in the new process assign stream numbers to ends of pipe */
-	    (void) dup2(fd[0][0],streams[2]);
-	    (void) dup2(fd[1][1],streams[3]);
-	    /* clean up on first 32 streams if they are open */
-	    for (i=3; i<32; i++)
-		  if (i != streams[2] && i != streams[3])
-			(void) close(i);
-	    /* start new process */
-	    execvp(process,my_argv);
-	    fprintf(stderr,"server: failed to exec %s process\n",process);
-	    _exit(-1);
-	  default:
-	    streams[0] = fd[1][0];
-	    streams[1] = fd[0][1];
-	    (void) close(fd[0][0]);
-	    (void) close(fd[1][1]);
-	    return(pid);
-      }
+	    free(fd);
+	    return 0;
+	}
+	if (DBX_val >= 5)
+	    fprintf(stderr,
+		    "Created pipe with read end %d and write end %d.\n",
+		    fd[i][0], fd[i][1]);
+    }
+
+    /* start process */
+    switch(pid = fork1()) {
+    case -1:
+	fprintf(stderr,"server: failed to fork %s process\n",process);
+	for(i=0; i<ns; i++) {
+	    close(fd[i][0]);
+	    close(fd[i][1]);
+	}
+	free(fd);
+	return 0;
+    case 0:
+	for(i=0; i<ns; i++)
+	{
+	    if (streams[i] >= 0) /* reading at child end */
+	    {
+		close(fd[i][1]);
+		dup2(fd[i][0], streams[i]);
+		if (DBX_val >= 5)
+		    fprintf(stderr, "child: fd %d moved to %d.\n", fd[i][0],
+			    streams[i]);
+	    }
+	    else /* writing at child end */
+	    {
+		close(fd[i][0]);
+		dup2(fd[i][1], -streams[i]);
+		if (DBX_val >= 5)
+		    fprintf(stderr, "child: fd %d moved to %d.\n", fd[i][1],
+			    -streams[i]);
+	    }
+	}
+	/* clean up streams if they are open */
+	for (i=3; i<getdtablesize(); i++)
+	{
+	    for(j=0; j<ns; j++)
+		if (i == abs(streams[j]))
+		    break;
+	    if (j == ns)
+		close(i);
+	}
+
+	/* start new process */
+	execvp(process,my_argv);
+	fprintf(stderr,"server: failed to exec %s process\n",process);
+	_exit(-1);
+    default:
+	for(i=0; i<ns; i++)
+	{
+	    if (streams[i] >= 0) /* writing at parent end */
+	    {
+		close(fd[i][0]);
+		streams[i] = fd[i][1];
+	    }
+	    else /* reading at parent end */
+	    {
+		close(fd[i][1]);
+		streams[i] = fd[i][0];
+	    }
+	}
+	free(fd);
+	if (DBX_val >= 5)
+	{
+	    fprintf(stderr, "pipes returned in streams:");
+	    for(i = 0; i < ns; i++)
+		fprintf(stderr, " %d", streams[i]);
+	    putc('\n', stderr);
+	}
+	return pid;
+    }
 /*      return(-1); */
 
 }
 
-/**************************************************************************************************/
+/********************************************************************/
 int
-start_gui(
+start_gui
 #if NeedFunctionPrototype
-		char *process, int argc, char **argv)
+(char *process, int argc, char **argv)
 #else
-      process, argc, argv)
-      char  *process;
-      int   argc;
-      char  **argv;
+(process, argc, argv)
+char  *process;
+int   argc;
+char  **argv;
 #endif
 {
-      char *my_argv[100];
-      char pid_str[24], dbx_str[24];
-      int streams[4];
-      int pid,i;
+    char *my_argv[100];
+    char pid_str[24], dbx_str[24];
+    int streams[2];
+    int pid,i;
+    
+    /* copy argument list */
+    my_argv[0] = process;
+    for(i=1; i<argc && i<91; i++)
+	my_argv[i] = argv[i];
+    my_argv[i] = "-sort_pid";
+    sprintf(pid_str,"%d",(int) getpid());
+    my_argv[i+1] = pid_str;
+    my_argv[i+2] = "-sdir";
+    my_argv[i+3] = tmp_spectrum_dir;
+    my_argv[i+4] = "-dbx";
+    sprintf(dbx_str,"%d",DBX_val);      
+    my_argv[i+5] = dbx_str;
+    my_argv[i+6] = "-sproc";
+    my_argv[i+7] = sunsort_proc;
+    my_argv[i+8] = (char *)0;
+    
+    /* set up stream number for new process*/
+    streams[0] = STDIN_FILENO;
+    streams[1] = -STDOUT_FILENO;
+    
+    /* start new process */      
+    pid = do_pipe_process(process, my_argv, 2, streams);
+    
+    if (pid > 0) {
+	if (waitpid(pid, NULL, WNOHANG) != pid) {
+	    FDIN = streams[1];
+	    FDDAT = FDOUT = streams[0];
+	    return(pid);
+	}
+    }
 
-/* copy argument list */
-      my_argv[0] = process;
-      for(i=1; i<argc && i<91; i++)
-	    my_argv[i] = argv[i];
-      my_argv[i] = "-sort_pid";
-      sprintf(pid_str,"%d",(int) getpid());
-      my_argv[i+1] = pid_str;
-      my_argv[i+2] = "-sdir";
-      my_argv[i+3] = tmp_spectrum_dir;
-      my_argv[i+4] = "-dbx";
-      sprintf(dbx_str,"%d",DBX_val);      
-      my_argv[i+5] = dbx_str;
-      my_argv[i+6] = "-sproc";
-      my_argv[i+7] = sunsort_proc;
-      my_argv[i+8] = (char *)0;
-
-/* set up stream number for new process*/
-      streams[2] = STDIN_FILENO;
-      streams[3] = STDOUT_FILENO;
-
-/* start new process */      
-      pid = do_pipe_process(process, my_argv, streams);
-
-      if (pid > 0) {
-	    if (waitpid(pid, NULL, WNOHANG) != pid) {
-		  FDIN = streams[0];
-		  FDDAT = FDOUT = streams[1];
-		  return(pid);
-	    }
-      }
-      
-      if (pid != 0) {
-	    (void) close(streams[0]);
-	    (void) close(streams[1]);
-      }
-
-      return(-1);
+    return(-1);
 }
 /**************************************************************************************************/
 int
-start_sortproc(
+start_sortproc
 #if NeedFunctionPrototype
-		char *process, int argc, char **argv)
+(char *process, int argc, char **argv)
 #else
-      process, argc, argv)
-      char  *process;
-      int   argc;
-      char  **argv;
+(process, argc, argv)
+char  *process;
+int   argc;
+char  **argv;
 #endif
 {
-      char *my_argv[100];
-      char dbx_str[24];
-      int streams[4];
-      int pid,i;
-
-/* copy argument list */
-      my_argv[0] = process;
-      for(i=1; i<argc && i<94; i++)
-	    my_argv[i] = argv[i];
-      my_argv[i] = "-sdir";
-      my_argv[i+1] = tmp_spectrum_dir;
-      my_argv[i+2] = "-dbx";
-      sprintf(dbx_str,"%d",DBX_val);       
-      my_argv[i+3] = dbx_str;
-      my_argv[i+4] = (char *)0;
-
-
-/* set up stream number for new process*/
-      streams[2] = SORT_SERVER_INFD;
-      streams[3] = SORT_SERVER_OUTFD;
-
-/* start new process */      
-      pid = do_pipe_process(process, my_argv, streams);
-
-      if (pid > 0) {
-	    if (waitpid(pid, NULL, WNOHANG) != pid) {
-		  sort_proc_OK = SORT_TRUE;
-		  FDSIN = streams[0];
-		  FDSOUT = streams[1];
-		  return(pid);
-	    }
-      }
-      
-      if (pid != 0) {
-	    (void) close(streams[0]);
-	    (void) close(streams[1]);
-      }
-
-      return(-1);
+    char *my_argv[100];
+    char dbx_str[24];
+    int streams[3];
+    int pid,i;
+    
+    /* copy argument list */
+    my_argv[0] = process;
+    for(i=1; i<argc && i<94; i++)
+	my_argv[i] = argv[i];
+    my_argv[i] = "-sdir";
+    my_argv[i+1] = tmp_spectrum_dir;
+    my_argv[i+2] = "-dbx";
+    sprintf(dbx_str,"%d",DBX_val);       
+    my_argv[i+3] = dbx_str;
+    my_argv[i+4] = (char *)0;
+    
+    /* set up stream number for new process*/
+    streams[0] = SORT_SERVER_INFD;
+    streams[1] = -SORT_SERVER_OUTFD;
+    streams[2] = STDIN_FILENO;
+    
+    /* start new process */      
+    pid = do_pipe_process(process, my_argv, 3, streams);
+    
+    if (pid > 0) {
+	if (waitpid(pid, NULL, WNOHANG) != pid) {
+	    sort_proc_OK = SORT_TRUE;
+	    FDSOUT = streams[0];
+	    FDSIN = streams[1];
+	    FDSUSEROUT = streams[2];
+	    return(pid);
+	}
+    }
+    
+    return(-1);
 }
 /**************************************************************************************************/
 
@@ -514,6 +622,7 @@ rm_leading_blanks(
 }
 
 /**************************************************************************************************/
+/*ARGSUSED*/
 void
 read_args(
 #if NeedFunctionPrototype
@@ -567,38 +676,54 @@ read_args(
 void
 main_event_loop(
 #if NeedFunctionPrototype
-	  void
+void
 #endif
-		)
+)
 {
-      char     buf[BUFSIZ];
-      char     reply[BUFSIZ];
-      int      fd;
-      int      ret_code;
-
-      for(;;) {
-	    /* check for data on input stream */
-	    if ( (fd = check_input()) != -1) 
-		  ret_code = read_input(fd, buf);
-	    else
-		  ret_code = SORT_FALSE;
-
-	    /* check status of child processes (if any) */
-	    check_children();
-
-	    /* if command has been read ... try and perform it */
-	    if (ret_code == SORT_TRUE) {
-		  if ( (ret_code = decode_command(buf,reply)) != SORT_FAIL) {
-			write_reply(ret_code,reply,fd);
-		  }
-		  /* end batch processing if error was detected in command from batch stream */
-		  if (batch_mode == SORT_TRUE && fd == FDBAT && ret_code != SORT_TRUE ) {
-			sprintf(buf,"failed batch command");
-			end_batch(buf,reply);
-		  }
+    char     buf[BUFSIZ];
+    char     reply[BUFSIZ];
+    int      fd, i;
+    int      ret_code;
+    
+    for(;;) {
+	/* check for data on input stream */
+	if ( (fd = check_input()) != -1) 
+	    ret_code = read_input(fd, buf);
+	else
+	    ret_code = SORT_FALSE;
+	
+	/* check status of child processes (if any) */
+	check_children();
+	
+	/* if command has been read ... try and perform it */
+	if (ret_code == SORT_TRUE) {
+	    /* If sorting data, pass command to sort process */
+	    if (fd == STDIN_FILENO && Sorting_data(0) != SORT_FALSE &&
+		sort_proc_OK == SORT_TRUE && *buf != '$')
+	    {
+		i = strlen(buf);
+		buf[i] = '\n';
+		if (write(FDSUSEROUT, buf, i+1) != i+1)
+		    fprintf(stderr, "server: attempt to communicate with "
+			    "sort process failed.\n");
 	    }
-
-      }
+	    else
+	    {
+		if ( (ret_code = decode_command(buf,reply)) != SORT_FAIL) {
+		    write_reply(ret_code,reply,fd);
+		}
+		/*
+		 * end batch processing if error was detected in command from
+		 * batch stream
+		 */
+		if (batch_mode == SORT_TRUE && fd == FDBAT &&
+		    ret_code != SORT_TRUE ) {
+		    sprintf(buf,"failed batch command");
+		    end_batch(buf,reply);
+		}
+	    }
+	}
+    }
 }
 
 int
@@ -619,7 +744,7 @@ check_input(
       FD_ZERO(&rset);
       FD_SET(STDIN_FILENO,&rset);
       FD_SET(FDIN,&rset);
-      if (Sorting_data(NULL) == SORT_FALSE)
+      if (Sorting_data(0) == SORT_FALSE)
 	    FD_SET(FDBAT,&rset);
       
 /* set up timer */
@@ -638,7 +763,7 @@ check_input(
 		  return(FDIN);
 	    else if (FD_ISSET(STDIN_FILENO,&rset))
 		  return(STDIN_FILENO);
-	    else if (FD_ISSET(FDBAT,&rset) && Sorting_data(NULL) != SORT_TRUE) 
+	    else if (FD_ISSET(FDBAT,&rset) && Sorting_data(0) != SORT_TRUE) 
 			return(FDBAT);
       }
       else {
@@ -675,10 +800,11 @@ read_input(
 	    }
 	    else {
 		  /* assume that input is dead and reset input to tty */
+                  fprintf(stderr, "DEBUG: EOF on fd %d\n", fd);
 		  if (batch_mode == SORT_TRUE) {
 			end_batch("EOF detected",buf);
 		  }
-		  else if (nogui == SORT_FALSE) {
+		  else if (fd == FDIN && nogui == SORT_FALSE) {
 			FDIN = STDIN_FILENO;
 			FDDAT = FDOUT = STDOUT_FILENO;
 		  }
@@ -751,8 +877,12 @@ check_children(
 			(void) close(FDSOUT);
 			FDSOUT = STDOUT_FILENO;
 		  }
+		  if (FDSUSEROUT != STDOUT_FILENO) {
+			(void) close(FDSUSEROUT);
+			FDSUSEROUT = STDOUT_FILENO;
+		  }
 		  /* check to see whether there is a sort thread alive */
-		  if (Sorting_data(NULL) == SORT_TRUE) {
+		  if (Sorting_data(0) == SORT_TRUE) {
 			stop_sort_threads();
 		  }
 		  fprintf(stderr,
@@ -813,6 +943,7 @@ static struct {
       {"debugsort",debugsort,"debugsort"},
       {"display1d", display_1d, "display1d <#1d spectrum nos> + up to 3 others"},
       {"display2d", display_2d, "display2d <#2d spectrum nos>"},
+      {"displayaction", display_action, "displayaction <display/mstick>"},
       {"editsort", editsort, "editsort <editor> <sort program name>"},
       {"eject", eject_func, "eject [access mode = r/w]"},
       {"end", end_batch, "end [batch]"},
@@ -846,72 +977,80 @@ static struct {
       {"viewwin", viewwindow, "viewwin <#window nos> <inside/outside> <#2d spectrum nos>"},
       {"wclose", close_filt_file, "wclose [tape/disk]"},
       {"wopen", open_filt_file, "wopen <tape/disk> <write file name>"},
+      {"wrsort", write_sort, "wrsort <message to send to sort process>"},
       { NULL, NULL, NULL}
 };
 /**************************************************************************************************/
 int
-decode_command(
+decode_command
 #if NeedFunctionPrototype
-	  char *ipbuf, char *opbuf)
+(char *ipbuf, char *opbuf)
 #else
-      ipbuf, opbuf)
-      char     *ipbuf;
-      char     *opbuf;      
+(ipbuf, opbuf)
+char     *ipbuf;
+char     *opbuf;      
 #endif
 {
-      char   com_str[BUFSIZ];
-      char   *opts;
-      char   *charptr;
-      int    i;
+    char   com_str[BUFSIZ];
+    char   *opts;
+    char   *charptr;
+    int    i;
 
-/* remove any blanks from the beginning of the command line */
-      ipbuf = rm_leading_blanks(ipbuf,BUFSIZ);
+    if (*ipbuf == '$')
+	ipbuf++;
+
+    /* remove any blanks from the beginning of the command line */
+    ipbuf = rm_leading_blanks(ipbuf,BUFSIZ);
       
-/* begin to decode command line	*/	      
-      if (sscanf(ipbuf,"%s",com_str) <= 0) {
-	    /* check that command line is not just a CR */      
-	    if (ipbuf[0] == '\n' || ipbuf[0] == '\0') {
-		  fprintf(stderr,"Sunsort > ");
-		  sprintf(opbuf,"wibble > \n");
-		  return(SORT_FAIL);
+    /* begin to decode command line	*/	      
+    if (sscanf(ipbuf,"%s",com_str) <= 0) {
+	/* check that command line is not just a CR */      
+	if (ipbuf[0] == '\n' || ipbuf[0] == '\0') {
+	    fprintf(stderr,"Sunsort > ");
+	    sprintf(opbuf,"wibble > \n");
+	    return(SORT_FAIL);
+	}
+	else
+	    sprintf(opbuf,"error decoding command string %s\n",ipbuf);
+	return(SORT_FALSE);
+    }
+    
+    opts = ipbuf + strlen(com_str);
+    /* convert command string to lower case */
+    for (charptr = com_str; *charptr != '\0'; charptr++)
+	if (isupper((int) *charptr)) tolower((int) *charptr);
+    if (DBX_val >= 5)
+	fprintf(stderr,"server: command string = %s, opts = %s\n",
+		com_str, opts);
+    
+    /* check that command line is a comment line */      
+    if (com_str[0] == '#')
+	return(comment_line(ipbuf+1,opbuf));
+    
+    /* using command call appropriate function to act on it */      
+    for(i=0; commands[i].string != NULL ; i++) {
+	if (strcmp(com_str,commands[i].string) == 0) {
+	    if (commands[i].func != NULL)
+		return((*commands[i].func)(opts, opbuf));
+	    else {
+		fprintf(stderr,"server: null command operation ... "
+			"seek help!\n");
+		return(SORT_FAIL);
 	    }
-	    else
-		  sprintf(opbuf,"error decoding command string %s\n",ipbuf);
-	    return(SORT_FALSE);
-      }
-      
-      opts = ipbuf + strlen(com_str);
-/* convert command string to lower case */
-      for (charptr = com_str; *charptr != '\0'; charptr++)
-	    if (isupper((int) *charptr)) tolower((int) *charptr);
-      if (DBX_val >= 5)
-	    fprintf(stderr,"server: command string = %s, opts = %s\n",com_str,opts);
-
-/* check that command line is a comment line */      
-      if (com_str[0] == '#')
-	    return(comment_line(ipbuf+1,opbuf));
-      
-/* using command call appropriate function to act on it */      
-      for(i=0; commands[i].string != NULL ; i++) {
-	    if (strcmp(com_str,commands[i].string) == 0) {
-		  if (commands[i].func != NULL)
-			return((*commands[i].func)(opts, opbuf));
-		  else {
-			fprintf(stderr,"server: null command operation ... seek help!\n");
-			return(SORT_FAIL);
-		  }
-	    }
-      }
-
-      fprintf(stderr,"server: unrecognised command << %s >> passing it to sh\n",com_str);
-      if (system(ipbuf) == 127) {
-	    fprintf(stderr,"server: could not execute shell\n");
-      }
-      else
-	    fprintf(stderr,"SunSort > ");
-      return(SORT_FAIL);
+	}
+    }
+    
+    fprintf(stderr, "server: unrecognised command << %s >> passing it to sh\n",
+	    com_str);
+    if (system(ipbuf) == 127) {
+	fprintf(stderr,"server: could not execute shell\n");
+    }
+    else
+	fprintf(stderr,"SunSort > ");
+    return(SORT_FAIL);
 }
-/**************************************************************************************************/
+
+/**************************************************************************/
 /*
  *  display help information
  */
@@ -960,98 +1099,112 @@ help_func(
 /*
  *  report the state of the sort process
  */
+/*ARGSUSED*/
 int
-status_func(
+status_func
 #if NeedFunctionPrototype
-	   char *opt_str, char *reply)
+(char *opt_str, char *reply)
 #else
-      opt_str, reply)
-      char     *opt_str;
-      char     *reply;
+(opt_str, reply)
+char     *opt_str;
+char     *reply;
 #endif
 {
-      sort_IO_t *ptr  = sort_io_data;
-      int       state = ptr->IO_state;
+    sort_IO_t *ptr  = sort_io_data;
+    int       state = ptr->IO_state;
+    
+    fprintf(stderr, "******************************************************"
+	    "******************\n");
+    /* reading information */      
+    if ( (state & R_TAPE_MOUNTED) && !(state & READING_TAPE)) {
+	(void) dtstate(0);
+    }
+    switch (state & READING_IN) {
+    case READING_DISK:
+	(void) ddstate(0);
+	fprintf(stderr, "Records read in so far = %d\n\n",
+		(int) *sorted_records); 
+	break;
+    case READING_TAPE:
+	(void) dtstate(0);
+	fprintf(stderr, "Records read in so far = %d\n\n",
+		(int) *sorted_records);
+	break;
+    case READING_REMOTE:
+	fprintf(stderr,"Reading from remote source\n");
+	(void) remote_op(STATUS_REMOTE);
+	break;
+    case READING_DEMON:
+	demonstatus();
+	fprintf(stderr, "Records read in so far = %d\n\n",
+		(int) *sorted_records); 
+	break;
+    default:
+	fprintf(stderr, "No read file open\n\n");
+	break;
+    }
 
-      fprintf(stderr,"*********************************************************\n");
-/* reading information */      
-      if ( (state & R_TAPE_MOUNTED) && !(state & READING_TAPE)) {
-	    (void) dtstate(0);
-      }
-      switch (state & READING_IN) {
-	    case READING_DISK:
-	    (void) ddstate(0);
-	    fprintf(stderr,"Records read in so far = %d\n\n",(int) *sorted_records); 
-	    break;
-	    case READING_TAPE:
-	    (void) dtstate(0);
-	    fprintf(stderr,"Records read in so far = %d\n\n",(int) *sorted_records);
-	    break;
-	    case READING_REMOTE:
-	    fprintf(stderr,"Reading from remote source\n");
-	    (void) remote_op(STATUS_REMOTE);
-	    break;
-	    case READING_DEMON:
-	    demonstatus();
-	    fprintf(stderr,"Records read in so far = %d\n\n",(int) *sorted_records); 
-	    break;
-	    default:
-	    fprintf(stderr,"No read file open\n\n");
-	    break;
-      }
-
-/* writing information */
-      if ( (state & W_TAPE_MOUNTED) && !(state & WRITING_TAPE)) {
-	    (void) dtstate(1);
-      }
-      switch (state & WRITING_OUT) {
-	    case WRITING_DISK:
-	    (void) ddstate(1);
-	    fprintf(stderr,"Records filtered out so far = %d\n\n",(int) *filtered_records);
-	    break;
-	    case WRITING_TAPE:
-	    (void) dtstate(1);
-	    fprintf(stderr,"Records filtered out so far = %d\n\n",(int) *filtered_records); 
-	    break;
-	    default:
-	    fprintf(stderr,"No write file open\n\n");
-	    break;
-      }
+    /* writing information */
+    if ( (state & W_TAPE_MOUNTED) && !(state & WRITING_TAPE)) {
+	(void) dtstate(1);
+    }
+    switch (state & WRITING_OUT) {
+    case WRITING_DISK:
+	(void) ddstate(1);
+	fprintf(stderr, "Records filtered out so far = %d\n\n",
+		(int) *filtered_records);
+	break;
+    case WRITING_TAPE:
+	(void) dtstate(1);
+	fprintf(stderr, "Records filtered out so far = %d\n\n",
+		(int) *filtered_records); 
+	break;
+    default:
+	fprintf(stderr, "No write file open\n\n");
+	break;
+    }
       
-      fprintf(stderr,"Last save to directory => %s \nLast load from directory => %s\n\n",
-	      dirs_io_data->save_dir,dirs_io_data->load_dir);
+    fprintf(stderr, "Last save to directory => %s \nLast load from "
+	    "directory => %s\n\n", dirs_io_data->save_dir,
+	    dirs_io_data->load_dir);
 
-      if (sort_proc_OK == SORT_TRUE) {
-	    fprintf(stderr,"sort process is running\n");
-	    fprintf(stderr,"sort process name = %s: pid = %ld\n", sunsort_proc,sort_pid);
-	    fprintf(stderr,"sort process to use decoder function named => %s\n\n",ptr->decoder);
-      }
-      else {
-	    fprintf(stderr,"sort process is NOT running\n\n");
-      }
-      if (nogui != SORT_TRUE) {
-	    fprintf(stderr,"gui process is running: pid = %ld\n\n",gui_pid);
-      }
-      else {
-	    fprintf(stderr,"gui process is NOT running\n\n");
-      }
-      if (graf_proc1_OK == SORT_TRUE) {
-	    fprintf(stderr,"1d graphics process is running: pid = %ld\n",graf_1d_pid);
-      }
-       else {
-	    fprintf(stderr,"1d graphics process is NOT running\n");
-      }
-      if (graf_proc2_OK == SORT_TRUE) {
-	    fprintf(stderr,"2d graphics process is running: pid = %ld\n",graf_2d_pid);
-      }
-      else {
-	    fprintf(stderr,"2d graphics process is NOT running\n");
-      }      
-
-      fprintf(stderr,"*********************************************************\n");
-      sprintf(reply,"\n");
-
-      return(SORT_TRUE);
+    if (sort_proc_OK == SORT_TRUE) {
+	fprintf(stderr, "sort process is running\n");
+	fprintf(stderr, "sort process name = %s: pid = %d, fds = %d, %d, %d\n",
+		sunsort_proc, sort_pid, FDSIN, FDSOUT, FDSUSEROUT);
+	fprintf(stderr, "sort process to use decoder function named => %s\n\n",
+		ptr->decoder);
+    }
+    else {
+	fprintf(stderr,"sort process is NOT running\n\n");
+    }
+    if (nogui != SORT_TRUE) {
+	fprintf(stderr, "gui process is running: pid = %d, fds = %d, %d\n\n",
+		gui_pid, FDIN, FDOUT);
+    }
+    else {
+	fprintf(stderr,"gui process is NOT running\n\n");
+    }
+    if (graf_proc1_OK == SORT_TRUE) {
+	fprintf(stderr, "1d graphics process is running: pid = %d\n",
+		graf_1d_pid);
+    }
+    else {
+	fprintf(stderr,"1d graphics process is NOT running\n");
+    }
+    if (graf_proc2_OK == SORT_TRUE) {
+	fprintf(stderr, "2d graphics process is running: pid = %d\n",
+		graf_2d_pid);
+    }
+    else {
+	fprintf(stderr, "2d graphics process is NOT running\n");
+    }      
+    
+    fprintf(stderr,"*******************************************************"
+	    "*****************\n");
+    sprintf(reply,"\n");
+    
+    return(SORT_TRUE);
 }
 
 /*
@@ -1100,7 +1253,7 @@ loadsort(
 	    fprintf(stderr,"server: loadsort function called with args => %s\n",opt_str);
       
 /* check that not trying to sort */      
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -1154,17 +1307,18 @@ loadsort(
 	    sort_proc_OK = SORT_FALSE;
 	    FDSIN = STDIN_FILENO;
 	    FDSOUT = STDOUT_FILENO;
+	    FDSUSEROUT = STDOUT_FILENO;
 	    sprintf(reply,"failed to load new sort process %s\n",sunsort_proc);
 	    return(SORT_FALSE);
       }
-
+      
       sprintf(reply,"loadsort started %s\n",sunsort_proc);
 
       if(keep_opt == SORT_FALSE)
-          fprintf(stderr, "server: spectra erased.\n");
+          fprintf(stderr,"server: Sort Loaded with new spectra\n");
       else
-          fprintf(stderr, "server: spectra preserved.\n");
-
+          fprintf(stderr,"server: Sort Loaded keeping old spectra\n");
+     
       return(SORT_TRUE);
 }
 
@@ -1190,7 +1344,7 @@ makesort(
 	    fprintf(stderr,"server: makesort function called with args => %s\n",opt_str);
 
 /* check that not trying to sort */      
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
@@ -1276,6 +1430,10 @@ killsort(
 		  (void) close(FDSOUT);
 		  FDSOUT = STDOUT_FILENO;
 	    }
+	    if (FDSUSEROUT != STDOUT_FILENO) {
+		  (void) close(FDSUSEROUT);
+	          FDSUSEROUT = STDOUT_FILENO;
+	    }
       }
       else {
 	    if (reset == SORT_TRUE)
@@ -1285,7 +1443,7 @@ killsort(
       }
 
 /* check to see whether we are sorting ... if so reset buffers etc */      
-      if (Sorting_data(NULL) == SORT_TRUE) {
+      if (Sorting_data(0) == SORT_TRUE) {
 	    stop_sort_threads();
       }
       else if (reset == SORT_TRUE) {
@@ -1300,49 +1458,57 @@ killsort(
  *  start debugger on current sort process
  */
 int
-debugsort(
+debugsort
 #if NeedFunctionPrototype
-	   char *opt_str, char *reply)
+(char *opt_str, char *reply)
 #else
-      opt_str, reply)
-      char     *opt_str;
-      char     *reply;
+(opt_str, reply)
+char     *opt_str;
+char     *reply;
 #endif
 {
-      char   *data;
+    char   *data;
 #ifdef SVR4
-      char  *debugger = "debugger";
-      extern void *background_thread(void *data);
+    char  *debugger = "debugger";
 #else
-      char  *debugger = "dbxtool";
-      extern void *background_thread();
+    char  *debugger = "dbxtool";
 #endif
-      
-      if (DBX_val >= 5)
-	    fprintf(stderr,"server: debugsort function called with args => %s\n",opt_str);
-      
-/* check that sort process is alive */
-      if (sort_proc_OK != SORT_TRUE) {
-	    sprintf(reply,"no sort process running ... perform loadsort first\n");
-	    return(SORT_FALSE);
-      }
 
-/* allocate memory for argument to debug_thread */      
-      data = (char *) malloc(strlen(debugger)+strlen(sunsort_proc)+24);
-      if (data == NULL) {
-	    sprintf(reply,"server: memory allocation failed in debugsort call\n");
-	    return(SORT_FALSE);
-      }
-      (void) sprintf(data,"%s %s %ld",debugger, sunsort_proc, sort_pid);
+    /* Should really search path for this, but that's too muck like hard
+       work */
 
-/* start a new thread to fork debugger */      
-      if (thr_create(NULL, NULL, background_thread, (void *) data, THR_DETACHED, NULL )) {
-	    sprintf(reply,"error while trying to start debug thread\n");
-	    return(SORT_FALSE);
-      }
+    char *workshop = "/opt/SUNWspro/bin/workshop";
+    
+    if (DBX_val >= 5)
+	fprintf(stderr,"server: debugsort function called with args => %s\n",
+		opt_str);
+    
+    /* check that sort process is alive */
+    if (sort_proc_OK != SORT_TRUE) {
+	sprintf(reply,"no sort process running ... perform loadsort first\n");
+	return(SORT_FALSE);
+    }
+    
+    if (access(workshop, X_OK) == 0)
+	debugger = workshop;
 
-      sprintf(reply,"debugsort started ok\n");
-      return(SORT_TRUE);
+    /* allocate memory for argument to debug_thread */      
+    data = (char *) malloc(strlen(debugger)+strlen(sunsort_proc)+24);
+    if (data == NULL) {
+	sprintf(reply,"server: memory allocation failed in debugsort call\n");
+	return(SORT_FALSE);
+    }
+    (void) sprintf(data,"%s %s %ld",debugger, sunsort_proc, sort_pid);
+    
+    /* start a new thread to fork debugger */      
+    if (thr_create(NULL, 0, background_thread, (void *) data, THR_DETACHED,
+		   NULL )) {
+	sprintf(reply,"error while trying to start debug thread\n");
+	return(SORT_FALSE);
+    }
+    
+    sprintf(reply,"debugsort started ok\n");
+    return(SORT_TRUE);
 }
 
 /*
@@ -1359,11 +1525,7 @@ editsort(
 #endif
 {
       char *data;
- #ifdef SVR4
-      extern void *background_thread(void *data);
-#else
-      extern void *background_thread();
-#endif     
+
       if (DBX_val >= 5)
 	    fprintf(stderr,"server: editsort function called with args => %s\n",opt_str);
 
@@ -1376,7 +1538,7 @@ editsort(
       (void) strcpy(data, opt_str);
       
 /* start a new thread to fork editor */      
-      if (thr_create(NULL, NULL, background_thread, (void *) data, THR_DETACHED, NULL )) {
+      if (thr_create(NULL, 0, background_thread, (void *) data, THR_DETACHED, NULL )) {
 	    sprintf(reply,"error while trying to start edit thread\n");
 	    return(SORT_FALSE);
       }
@@ -1404,7 +1566,7 @@ loadspec(
 	    fprintf(stderr,"server: loadspec function called with args => %s\n",opt_str);
 
 /* check that not trying to sort */      
-      if (Sorting_data(NULL) != SORT_FALSE) {
+      if (Sorting_data(0) != SORT_FALSE) {
 	    sprintf(reply,"invalid command while sorting\n");
 	    return(SORT_FALSE);
       }
