@@ -8,7 +8,6 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
-#include <memory.h>
 #include <malloc.h>
 #include <signal.h>
 #include <stdarg.h>
@@ -20,30 +19,21 @@
 
 #include <time.h>
 #include <fcntl.h>
-#include <errno.h>
 
-#ifdef SVR4
-#define NeedFunctionPrototype 1
-#else
-#define NeedFunctionPrototype 0
-#endif
-
-#include "sort_thread.h"
+#include "sort_def.h"
+#include "sort_threadint.h"
+#include "sort_main.h"
+#include "data_disk.h"
+#include "data_tape.h"
+#include "data_remote.h"
 #include "demon.h"
 
 /* local variables */
 static struct shared_data *sdat_ptr;
 static char               *data_file;
 
-#if NeedFunctionPrototype
-typedef int (*IOfunc)(int, char *);
 static IOfunc       get_data_func = NULL;
 static IOfunc       put_data_func = NULL;
-#else
-typedef int (*IOfunc)();
-static IOfunc       get_data_func = NULL;
-static IOfunc       put_data_func = NULL;
-#endif
 
 /* variables shared with other source modules */
 int                sort_records = 0;
@@ -53,22 +43,6 @@ unsigned long      *filtered_records;
 sort_IO_t          *sort_io_data;
 dirs_IO_t          *dirs_io_data;
 
-extern int         nogui;
-extern int         DBX_val;
-
-
-#if NeedFunctionPrototype
-      extern IOfunc ddiofunc(int);
-      extern IOfunc dtiofunc(int);
-      extern int remoteget(int, char *);
-      extern pid_t gui_pid;
-#else
-      extern IOfunc ddiofunc();
-      extern IOfunc dtiofunc();
-      extern int remoteget();
-      extern int gui_pid;
-#endif
-
 /*
  *  create directory in which shared data file and spectra files shall reside
  */
@@ -89,7 +63,7 @@ create_shared_data(
       int i;
       
 /* create name of shared data file */
-      data_file = (char *) malloc(strlen(data_file_dir)+strlen(name));
+      data_file = (char *) malloc(strlen(data_file_dir)+strlen(name)+1);
       if (data_file == NULL) {
 	    perror("create_shared_data");
 	    fprintf(stderr,"server: failed to allocate memory for shared data file name\n");
@@ -111,6 +85,7 @@ create_shared_data(
 	    return(-1);
       }
       
+      /*LINTED*/
       if ( (caddr_t)(sdat_ptr = (struct shared_data *)mmap(0, SHDATA_SIZE, PROT_READ | PROT_WRITE,
 							MAP_SHARED, filedes, 0)) == (caddr_t) -1) {
 	    fprintf(stderr,"server: mmap error on shared data file\n");
@@ -404,7 +379,7 @@ Sorting_data(
 	    perror("mutex sorting_data lock");
 	    return(SORT_FAIL);
       }
-      if (set_flag == NULL) {
+      if (set_flag == 0) {
 	    retcode = sorting_data;
       }
       else {
@@ -437,7 +412,7 @@ Stop_interupt(
       int            retcode;
       
       (void) mutex_lock(&stop_interupt_mp);
-      if (set_flag == NULL) {
+      if (set_flag == 0) {
 	    retcode = stop_interupt;
       }
       else {
@@ -454,145 +429,157 @@ Stop_interupt(
 /*
  * read data records from source medium and put in shared data area
  */
+/*ARGSUSED*/
 static void *
-read_thread(
+read_thread
 #if NeedFunctionPrototype
-		  void *data)
+(void *data)
 #else
-      data)
-      void *data;
+(data)
+void *data;
 #endif
 {
-      data_buf_t *dptr = &sdat_ptr->read_buf[0];
-      char command[FORMAT_NAME_LEN+12];
-      int buffer_pos = 0;
-      int records;
-      int bytes;
-      timestruc_t to;
-      to.tv_nsec = 0;
-
-      if (DBX_val >= 5)
-	    fprintf(stderr,"server: Starting read thread\n");
-
-/* skip requested number of records */
-      for (records = 1 ;records <= skip_records; records++) {
-	    bytes = (*get_data_func)(MAX_DATA_SIZE, (char *) dptr->data);
-	    if (bytes <= 0) {
-		  /* exit read thread */
-		  if (bytes == 0 && DBX_val < 1)
-			fprintf(stderr,"read on data source returned %d (eof = 0) at record %d\n",bytes,records);
-		  thr_exit(NULL);
-	    }
-      }
-      
-/* set sorting state to ON */
-      (void) mutex_lock(&sdat_ptr->mp_sorting);
-      sdat_ptr->sorting = SORTING_ON;
-      (void) mutex_unlock(&sdat_ptr->mp_sorting);
-      
-/* tell sort process to start sorting */
-      if (sort_io_data->IO_state & WRITING_OUT) 
-	    (void) sprintf(command,"sortf %s %s",
-			   sort_io_data->decoder, sort_io_data->decoder);
-      else 
-	    (void) sprintf(command,"sort %s %s",
-			   sort_io_data->decoder, sort_io_data->decoder);
-      write_to_sortproc(command);
-      
-/* sort requested number of records */
-/* note:  a continue call before the very bottom of the loop does not advance the buffer position. */
-      for (records = 1 ; records <= sort_records && !Stop_interupt(NULL);
-	   dptr = &sdat_ptr->read_buf[buffer_pos]) {
-
-/* check state of next buffer in shared data storage */	    
-	    (void) mutex_lock(&dptr->mp);
-	    if (dptr->state == EMPTY_BUF) {
-		  dptr->state = FILLING_BUF;
-	    }
-	    else {
-		  to.tv_sec = time(NULL) + RTHREAD_WAIT;
-		  (void) cond_timedwait(&dptr->cv, &dptr->mp, &to);
-		  (void) mutex_unlock(&dptr->mp);
-		  continue;  
-	    }
-	    (void) mutex_unlock(&dptr->mp);
-
-/* copy block into shared memory buffer */
-	    bytes = (*get_data_func)(MAX_DATA_SIZE, (char *) dptr->data);
-	    if (bytes <= 0) {
-		  fprintf(stderr,"read on data source returned %d (eof = 0) at record %d\n",
-			  bytes,records+skip_records);
-		  break;
-	    }
-	    dptr->data_size = bytes;
-    	    records++;
-	    
-/* update state of next buffer in shared data storage */	  
-	    (void) mutex_lock(&dptr->mp);
-	    if (dptr->state == FILLING_BUF) {
-		  dptr->state = FULL_BUF;
-		  (void) cond_signal(&dptr->cv);
-	    }
-	    else {
-		  dptr->state = END_OF_DATA_BUF;
-		  (void) mutex_unlock(&dptr->mp);
-/*		  (void) Stop_interupt(SORT_SET,SORT_EXIT); */
-		  fprintf(stderr,"server: data buffer in wrong state, junking record %d\n",records+skip_records);
-		  break;
-	    }
-	    (void) mutex_unlock(&dptr->mp);
-	    
-	    buffer_pos = (buffer_pos+1) % NOS_OF_RBUFS;
-      }
-
-/* exit prematurely from thread if requested to do so */     
-      if (Stop_interupt(NULL) == SORT_EXIT) {
-	    if (DBX_val >= 5)
-		  fprintf(stderr,"server: Ending read thread prematurely after %d records\n",records-1);
+    data_buf_t *dptr = &sdat_ptr->read_buf[0];
+    char command[2*FORMAT_NAME_LEN+12];
+    int buffer_pos = 0;
+    int records;
+    int bytes;
+    timestruc_t to;
+    to.tv_nsec = 0;
+    
+    if (DBX_val >= 5)
+	fprintf(stderr,"server: Starting read thread\n");
+    
+    /* skip requested number of records */
+    for (records = 1 ;records <= skip_records; records++) {
+	bytes = (*get_data_func)(MAX_DATA_SIZE, (char *) dptr->data);
+	if (bytes <= 0) {
+	    /* exit read thread */
+	    if (bytes == 0 && DBX_val < 1)
+		fprintf(stderr, "read on data source returned %d (eof = 0) "
+			"at record %d\n", bytes, records);
 	    thr_exit(NULL);
-      }
+	}
+    }
+      
+    /* set sorting state to ON */
+    (void) mutex_lock(&sdat_ptr->mp_sorting);
+    sdat_ptr->sorting = SORTING_ON;
+    (void) mutex_unlock(&sdat_ptr->mp_sorting);
+      
+    /* tell sort process to start sorting */
+    if (sort_io_data->IO_state & WRITING_OUT) 
+	(void) sprintf(command,"sortf %s %s",
+		       sort_io_data->decoder, sort_io_data->decoder);
+    else 
+	(void) sprintf(command,"sort %s %s",
+		       sort_io_data->decoder, sort_io_data->decoder);
+    write_to_sortproc(command);
+    
+    /* 
+     * sort requested number of records
+     * note: a continue call before the very bottom of the loop does not
+     * advance the buffer position.
+     */
+    for (records = 1 ; records <= sort_records && !Stop_interupt(0);
+	 dptr = &sdat_ptr->read_buf[buffer_pos]) {
 
-/* set current buffer to end of data state */
-      (void) mutex_lock(&dptr->mp);
-      while (dptr->state & (FULL_BUF|EMPTYING_BUF)) {
+	/* check state of next buffer in shared data storage */	    
+	(void) mutex_lock(&dptr->mp);
+	if (dptr->state == EMPTY_BUF) {
+	    dptr->state = FILLING_BUF;
+	}
+	else {
 	    to.tv_sec = time(NULL) + RTHREAD_WAIT;
 	    (void) cond_timedwait(&dptr->cv, &dptr->mp, &to);
-	    if (Stop_interupt(NULL) == SORT_EXIT) 
-		  break;
-      }
-      dptr->state = END_OF_DATA_BUF;
-      (void) cond_signal(&dptr->cv);
-      (void) mutex_unlock(&dptr->mp);
+	    (void) mutex_unlock(&dptr->mp);
+	    continue;  
+	}
+	(void) mutex_unlock(&dptr->mp);
 
-      if (DBX_val >= 5)
-	    fprintf(stderr,"server: read thread EOD buffer %d after %d records\n",buffer_pos,records-1);
+	/* copy block into shared memory buffer */
+	bytes = (*get_data_func)(MAX_DATA_SIZE, (char *) dptr->data);
+	if (bytes <= 0) {
+	    fprintf(stderr, "read on data source returned %d (eof = 0) at "
+		    "record %d\n", bytes, records+skip_records);
+	    break;
+	}
+	dptr->data_size = bytes;
+	records++;
+	    
+	/* update state of next buffer in shared data storage */	  
+	(void) mutex_lock(&dptr->mp);
+	if (dptr->state == FILLING_BUF) {
+	    dptr->state = FULL_BUF;
+	    (void) cond_signal(&dptr->cv);
+	}
+	else {
+	    dptr->state = END_OF_DATA_BUF;
+	    (void) mutex_unlock(&dptr->mp);
+	    (void) Stop_interupt(SORT_SET,SORT_EXIT);
+	    fprintf(stderr, "server: data buffer in wrong state, junking "
+		    "record %d\n", records+skip_records);
+	    break;
+	}
+	(void) mutex_unlock(&dptr->mp);
+	
+	buffer_pos = (buffer_pos+1) % NOS_OF_RBUFS;
+    }
+
+    /* exit prematurely from thread if requested to do so */     
+    if (Stop_interupt(0) == SORT_EXIT)
+    {
+	fprintf(stderr,
+		"server: Emergency stop of read thread at after %d records.\n",
+		records-1);
+	thr_exit(NULL);
+    }
+
+    /* set current buffer to end of data state */
+    (void) mutex_lock(&dptr->mp);
+    while (dptr->state & (FULL_BUF|EMPTYING_BUF)) {
+	to.tv_sec = time(NULL) + RTHREAD_WAIT;
+	(void) cond_timedwait(&dptr->cv, &dptr->mp, &to);
+	if (Stop_interupt(0) == SORT_EXIT) 
+	    break;
+    }
+    dptr->state = END_OF_DATA_BUF;
+    (void) cond_signal(&dptr->cv);
+    (void) mutex_unlock(&dptr->mp);
+    
+    if (DBX_val >= 5)
+	fprintf(stderr,"server: read thread EOD buffer %d after %d "
+		"records\n",buffer_pos,records-1);
       
-/* wait until sorting process has finished */
-      (void) mutex_lock(&sdat_ptr->mp_sorting);
-      while (sdat_ptr->sorting  == SORTING_ON) {
-	    to.tv_sec = time(NULL) + RTHREAD_WAIT;
-	    (void) cond_timedwait(&sdat_ptr->cv_sorting, &sdat_ptr->mp_sorting, &to);
-	    if (Stop_interupt(NULL) == SORT_EXIT) 
-		  break;
-      }
-      sdat_ptr->sorting = SORTING_OFF;
-      (void) mutex_unlock(&sdat_ptr->mp_sorting);
-
-      if (DBX_val >= 10)
-	    (void) diagnostic_R_buffers();
-
-/* exit read thread now */
-      (void) empty_R_buffers();
-      if (DBX_val >= 5) 
-	    fprintf(stderr,"server: Ending read thread\n");
-
-      thr_exit(NULL);
-      return(NULL);
+    /* wait until sorting process has finished */
+    (void) mutex_lock(&sdat_ptr->mp_sorting);
+    while (sdat_ptr->sorting  == SORTING_ON) {
+	to.tv_sec = time(NULL) + RTHREAD_WAIT;
+	(void) cond_timedwait(&sdat_ptr->cv_sorting, &sdat_ptr->mp_sorting,
+			      &to);
+	if (Stop_interupt(0) == SORT_EXIT) 
+	    break;
+    }
+    sdat_ptr->sorting = SORTING_OFF;
+    (void) mutex_unlock(&sdat_ptr->mp_sorting);
+    
+    if (DBX_val >= 10)
+	(void) diagnostic_R_buffers();
+    
+    /* exit read thread now */
+    (void) empty_R_buffers();
+    if (DBX_val >= 5) 
+	fprintf(stderr,"server: Ending read thread\n");
+    
+    thr_exit(NULL);
+    return(NULL);
 }
 
 /*
- *  read filtered data records from shared data source and write them to storage medium
+ * read filtered data records from shared data source and write them to
+ * storage medium
  */
+/*ARGSUSED*/
 static void *
 write_thread(
 #if NeedFunctionPrototype
@@ -612,7 +599,7 @@ write_thread(
       if (DBX_val >= 5)
 	    fprintf(stderr,"server: Starting write thread\n");
 
-      for(records=0; Stop_interupt(NULL) != SORT_EXIT; ) {
+      for(records=0; Stop_interupt(0) != SORT_EXIT; ) {
 	    dptr = &sdat_ptr->write_buf[buffer_pos];
 
 /* check state of next buffer in shared data storage */	    
@@ -772,14 +759,14 @@ sort_thread(
 /* and sort */
       if (filter) {
 	    /* start write thread if filtering data */
-	    if (thr_create(NULL, NULL, write_thread, data, NULL, &wthread)) {
+	    if (thr_create(NULL, 0, write_thread, data, 0, &wthread)) {
 		  (void) Sorting_data(SORT_SET, SORT_FALSE);
 		  fprintf(stderr,"server: error while trying to start write thread\n");
 		  sort_thr_exit(-5);
 	    }
       }
       
-      if (thr_create(NULL, NULL, read_thread, data, NULL, &rthread)) {
+      if (thr_create(NULL, 0, read_thread, data, 0, &rthread)) {
 	    (void) Sorting_data(SORT_SET, SORT_FALSE);
 	    fprintf(stderr,"server: error while trying to start read thread\n");
 	    if (filter)
@@ -815,7 +802,7 @@ stop_sort_threads(
 {
       (void) Stop_interupt(SORT_SET,SORT_EXIT);
 /* wait for sort thread to signal exit */      
-      while(Sorting_data(NULL) == SORT_FALSE)
+      while(Sorting_data(0) == SORT_FALSE)
 	    thr_yield();
       if (reset_all_buffers() == -1)
 	    fprintf(stderr,"server:  reset buffers in kill threads failed\n");
